@@ -273,6 +273,7 @@ rb_new_station_dialog_response_cb (GtkDialog *gtkdialog,
 				   RBNewStationDialog *dialog)
 {
 	const char *location;
+	char *trimmed_location;
 	GValue title_val = { 0, };
 	GValue genre_val = { 0, };
 	RhythmDBEntry *entry;
@@ -280,22 +281,27 @@ rb_new_station_dialog_response_cb (GtkDialog *gtkdialog,
 		return;
 
 	location = gtk_entry_get_text (GTK_ENTRY (dialog->priv->location));
+	trimmed_location = g_strstrip (g_strdup (location));
 
 	g_value_init (&title_val, G_TYPE_STRING);
-	g_value_set_string (&title_val, gtk_entry_get_text (GTK_ENTRY (dialog->priv->title)));
+	g_value_set_static_string (&title_val,
+				   gtk_entry_get_text (GTK_ENTRY (dialog->priv->title)));
 	g_value_init (&genre_val, G_TYPE_STRING);
-	g_value_set_string (&genre_val,
-			    gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (dialog->priv->genre)->entry)));
-	entry = rhythmdb_entry_lookup_by_location (dialog->priv->db, location);
-	if (!entry)
-		entry = rhythmdb_entry_new (dialog->priv->db,
-					    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
-					    location);
-	rhythmdb_entry_set (dialog->priv->db, entry, RHYTHMDB_PROP_TITLE, &title_val);
-	rhythmdb_entry_set (dialog->priv->db, entry, RHYTHMDB_PROP_GENRE, &genre_val);
-	rhythmdb_commit (dialog->priv->db);
+	g_value_set_static_string (&genre_val,
+				   gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (dialog->priv->genre)->entry)));
+	entry = rhythmdb_entry_lookup_by_location (dialog->priv->db, trimmed_location);
+	if (entry)
+		return;
+	entry = rhythmdb_entry_new (dialog->priv->db,
+				    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+				    trimmed_location);
+	rhythmdb_entry_set_uninserted (dialog->priv->db, entry, RHYTHMDB_PROP_TITLE, &title_val);
 	g_value_unset (&title_val);
+	rhythmdb_entry_set_uninserted (dialog->priv->db, entry, RHYTHMDB_PROP_GENRE, &genre_val);
 	g_value_unset (&genre_val);
+	rhythmdb_commit (dialog->priv->db);
+
+	g_free (trimmed_location);
 }
 
 static void
