@@ -37,35 +37,10 @@ rb_stock_icons_init (void)
 
 	static const char *items[] =
 	{
-		RB_STOCK_PLAY,
-		RB_STOCK_PAUSE,
-		RB_STOCK_STOP,
-		RB_STOCK_PREVIOUS,
-		RB_STOCK_NEXT,
-		RB_STOCK_REWIND,
-		RB_STOCK_SHUFFLE,
-		RB_STOCK_REPEAT,
-		RB_STOCK_VISUALS,
-		RB_STOCK_PLAYLIST,
-		RB_STOCK_PLAYLIST_MENU,
-		RB_STOCK_LIBRARY,
-		RB_STOCK_IRADIO,
-		RB_STOCK_PLAYING,
-		RB_STOCK_PAUSED,
-		RB_STOCK_ALBUM,
-		RB_STOCK_VOLUME_ZERO,
-		RB_STOCK_VOLUME_MIN,
-		RB_STOCK_VOLUME_MEDIUM,
-		RB_STOCK_VOLUME_MAX,
-		RB_STOCK_VOLUME_MUTE,
 		RB_STOCK_TRAY_ICON,
 		RB_STOCK_SET_STAR,
-		RB_STOCK_AUTOMATIC_PLAYLIST,
 		RB_STOCK_UNSET_STAR,
-		RB_STOCK_NO_STAR,
-		RB_STOCK_DND_ICON,
-                RB_STOCK_AUDIOCD,
-		RB_STOCK_IPOD
+		RB_STOCK_NO_STAR
 	};
 
 	g_return_if_fail (factory == NULL);
