@@ -20,7 +20,11 @@
  */
 
 #include "rb-util.h"
+#include <gtk/gtk.h>
 #include <string.h>
+#include <libgnomevfs/gnome-vfs.h>
+
+static GPrivate * private_is_primary_thread;
 
 gboolean
 rb_true_function (gpointer dummy)
@@ -34,6 +38,12 @@ rb_false_function (gpointer dummy)
 	return FALSE;
 }
 
+gpointer
+rb_null_function (gpointer dummy)
+{
+	return NULL;
+}
+
 int
 rb_gvalue_compare (GValue *a, GValue *b)
 {
@@ -166,6 +176,21 @@ rb_gvalue_compare (GValue *a, GValue *b)
 	return retval;
 }
 
+int
+rb_compare_gtimeval (GTimeVal *a, GTimeVal *b)
+{
+	if (a->tv_sec == b->tv_sec)
+		/* It's quite unlikely that microseconds are equal,
+		 * so just ignore that case, we don't need a lot
+		 * of precision.
+		 */
+		return a->tv_usec > b->tv_usec ? 1 : -1;
+	else if (a->tv_sec > b->tv_sec)
+		return 1;
+	else
+		return -1;
+}
+
 /* Taken from totem/video-utils.c CVS HEAD 2004-04-22 */
 static void
 totem_pixbuf_mirror (GdkPixbuf *pixbuf)
@@ -206,7 +231,8 @@ totem_pixbuf_mirror (GdkPixbuf *pixbuf)
 /* Same as gtk_image_new_from_stock except that it mirrors the icons for RTL 
  * languages
  */
-GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size)
+GtkWidget *
+rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size)
 {
 
 	if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_LTR) {
@@ -242,3 +268,115 @@ GtkWidget *rb_image_new_from_stock (const gchar *stock_id, GtkIconSize size)
 
 	return NULL;
 }
+
+void
+rb_gtk_action_popup_menu (GtkUIManager *uimanager, const char *path)
+{
+	GtkWidget *menu;
+
+	menu = gtk_ui_manager_get_widget (uimanager, path);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, 
+			gtk_get_current_event_time ());
+}
+
+static GList *
+get_mount_points (void)
+{
+	GnomeVFSVolumeMonitor *monitor;
+	GList *volumes;
+	GList *it;
+	GList *mount_points = NULL;
+
+	monitor = gnome_vfs_get_volume_monitor ();
+	/* FIXME: should also get the list of connected drivers (network
+	 * shares I assume)
+	 */
+	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (monitor);
+
+	for (it = volumes; it != NULL; it = it->next) {
+		gchar *uri;
+		GnomeVFSVolume *volume;
+
+		volume = GNOME_VFS_VOLUME (it->data);
+		uri = gnome_vfs_volume_get_activation_uri (volume);
+		g_assert (uri != NULL);
+		mount_points = g_list_prepend (mount_points, uri);
+	}
+
+	g_list_foreach (volumes, (GFunc)gnome_vfs_volume_ref, NULL);
+	g_list_free (volumes);
+
+	return mount_points;
+}
+
+
+gchar *
+rb_uri_get_mount_point (const char *uri)
+{
+	GList *mount_points = get_mount_points ();
+	GList *it;
+	gchar *mount_point = NULL;
+
+	for (it = mount_points; it != NULL; it = it->next) {
+		if (g_str_has_prefix (uri, it->data)) {
+			if ((mount_point == NULL) || (strlen (mount_point) < strlen (it->data))) {
+				g_free (mount_point);
+				mount_point = g_strdup (it->data);
+			}
+		}
+	}
+	g_list_foreach (mount_points, (GFunc)g_free, NULL);
+	g_list_free (mount_points);
+
+	return mount_point;
+}
+
+gboolean
+rb_uri_is_mounted (const char *uri)
+{
+	GList *mount_points = get_mount_points ();
+	GList *it;
+	gboolean found = FALSE;
+
+	if ((uri == NULL) || (*uri == '\0')) {
+		return TRUE;
+	}
+
+	for (it = mount_points; it != NULL; it = it->next) {
+		if (strcmp (it->data, uri) == 0) {
+			found = TRUE;
+			break;
+		}
+	}
+	g_list_foreach (mount_points, (GFunc)g_free, NULL);
+	g_list_free (mount_points);
+
+/*	if (found == FALSE) {
+		g_print ("%s not mounted\n", uri);
+		}*/
+
+	return found;
+}
+
+gboolean
+rb_is_main_thread (void)
+{
+	if (g_thread_supported()) {
+		return GPOINTER_TO_UINT(g_private_get (private_is_primary_thread)) == 1;
+	} else {
+		return TRUE;
+	}
+}
+
+
+void
+rb_threads_init (void)
+{
+	private_is_primary_thread = g_private_new (NULL);
+	g_private_set (private_is_primary_thread, GUINT_TO_POINTER (1));
+
+	/* not really necessary, but in case it does something besides
+	 * set up lock functions some day..
+	 */
+	gdk_threads_init ();
+}
