@@ -41,7 +41,6 @@
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnome/gnome-i18n.h>
-#include "rb-string-helpers.h"
 #include "rhythmdb-marshal.h"
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
@@ -58,6 +57,8 @@
 #define RB_PARSE_DISJ (xmlChar *) "disjunction"
 #define RB_PARSE_GREATER (xmlChar *) "greater"
 #define RB_PARSE_LESS (xmlChar *) "less"
+#define RB_PARSE_CURRENT_TIME_WITHIN (xmlChar *) "current-time-within"
+#define RB_PARSE_CURRENT_TIME_NOT_WITHIN (xmlChar *) "current-time-not-within"
 
 #define RB_PARSE_NICK_START (xmlChar *) "["
 #define RB_PARSE_NICK_END (xmlChar *) "]"
@@ -160,6 +161,7 @@ struct RhythmDBEvent
 	/* ENTRY_RESTORED / ENTRY_SET */
 	RhythmDBEntry *entry;
 	/* ENTRY_SET */
+	gboolean signal_change;
 	RhythmDBEntryChange change;
 };
 
@@ -182,8 +184,7 @@ static gpointer add_thread_main (struct RhythmDBAddThreadData *data);
 static gpointer action_thread_main (RhythmDB *db);
 static gpointer query_thread_main (struct RhythmDBQueryThreadData *data);
 static void queue_stat_uri (const char *uri, RhythmDB *db);
-static void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
-					 guint propid, GValue *value);
+static void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry, gboolean notify_if_inserted, guint propid, const GValue *value);
 static void rhythmdb_entry_set_mount_point (RhythmDB *db, 
  					    RhythmDBEntry *entry, 
  					    const gchar *realuri);
@@ -734,7 +735,6 @@ rhythmdb_get_readonly (RhythmDB *db)
 	return (g_atomic_int_get (&db->priv->read_counter) > 0);
 }
 
-
 static void
 rhythmdb_read_enter (RhythmDB *db)
 {
@@ -816,7 +816,8 @@ rhythmdb_commit_internal (RhythmDB *db, gboolean signal_changed)
 							 RHYTHMDB_PROP_LOCATION);
 			queue_stat_uri (uri, db);
 		}
-
+		g_assert (entry->inserted == FALSE);
+		entry->inserted = TRUE;
 	}
 
 	g_list_free (db->priv->added_entries);
@@ -875,6 +876,7 @@ rhythmdb_entry_allocate (RhythmDB *db, RhythmDBEntryType type)
 void
 rhythmdb_entry_insert (RhythmDB *db, RhythmDBEntry *entry)
 {
+	g_assert (entry->inserted == FALSE);
 	db->priv->added_entries = g_list_append (db->priv->added_entries, entry);	
 }
 
@@ -1018,7 +1020,7 @@ set_metadata_string_default_unknown (RhythmDB *db,
 		g_value_set_static_string (&val, unknown);
 	} else if (g_value_get_string (&val)[0] == '\0')
 		g_value_set_static_string (&val, unknown);
-	rhythmdb_entry_set_internal (db, entry, prop, &val);
+	rhythmdb_entry_set_internal (db, entry, TRUE, prop, &val);
 	g_value_unset (&val);
 }
 
@@ -1028,13 +1030,12 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 {
 	const char *mime;
 	GValue val = {0,};
-	GTimeVal time;
 
 	g_value_init (&val, G_TYPE_STRING);
 	mime = rb_metadata_get_mime (metadata);
 	if (mime) {
 		g_value_set_string (&val, mime);
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_MIMETYPE, &val);
 	}
 	g_value_unset (&val);
@@ -1046,7 +1047,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 		g_value_init (&val, G_TYPE_ULONG);
 		g_value_set_ulong (&val, 0);
 	}
-	rhythmdb_entry_set_internal (db, entry, 
+	rhythmdb_entry_set_internal (db, entry, TRUE,
 				     RHYTHMDB_PROP_TRACK_NUMBER, &val);
 	g_value_unset (&val);
 
@@ -1057,7 +1058,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 		g_value_init (&val, G_TYPE_ULONG);
 		g_value_set_ulong (&val, 0);
 	}
-	rhythmdb_entry_set_internal (db, entry,
+	rhythmdb_entry_set_internal (db, entry, TRUE,
 				     RHYTHMDB_PROP_DISC_NUMBER, &val);
 	g_value_unset (&val);
 
@@ -1065,7 +1066,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 	if (rb_metadata_get (metadata,
 			     RB_METADATA_FIELD_DURATION,
 			     &val)) {
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_DURATION, &val);
 		g_value_unset (&val);
 	}
@@ -1074,7 +1075,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 	if (rb_metadata_get (metadata,
 			     RB_METADATA_FIELD_BITRATE,
 			     &val)) {
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_BITRATE, &val);
 		g_value_unset (&val);
 	}
@@ -1082,7 +1083,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 	/* filesize */
 	g_value_init (&val, G_TYPE_UINT64);
 	g_value_set_uint64 (&val, vfsinfo->size);
-	rhythmdb_entry_set_internal (db, entry, RHYTHMDB_PROP_FILE_SIZE, &val);
+	rhythmdb_entry_set_internal (db, entry, TRUE, RHYTHMDB_PROP_FILE_SIZE, &val);
 	g_value_unset (&val);
 
 	/* title */
@@ -1098,21 +1099,13 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 		g_value_set_string (&val, utf8name);
 		g_free (utf8name);
 	}
-	rhythmdb_entry_set_internal (db, entry, RHYTHMDB_PROP_TITLE, &val);
+	rhythmdb_entry_set_internal (db, entry, TRUE, RHYTHMDB_PROP_TITLE, &val);
 	g_value_unset (&val);
 
 	/* mtime */
 	g_value_init (&val, G_TYPE_ULONG);
 	g_value_set_ulong (&val, vfsinfo->mtime);
-	rhythmdb_entry_set_internal (db, entry, RHYTHMDB_PROP_MTIME, &val);
-	g_value_unset (&val);
-
-        /* first seen and last seen */
-	g_get_current_time (&time);
-	g_value_init (&val, G_TYPE_ULONG);
-	g_value_set_ulong (&val, time.tv_sec);
-	rhythmdb_entry_set_internal (db, entry, RHYTHMDB_PROP_FIRST_SEEN, &val);
-	rhythmdb_entry_set_internal (db, entry, RHYTHMDB_PROP_LAST_SEEN, &val);
+	rhythmdb_entry_set_internal (db, entry, TRUE, RHYTHMDB_PROP_MTIME, &val);
 	g_value_unset (&val);
 
 	/* genre */
@@ -1133,7 +1126,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
         if (rb_metadata_get (metadata,
                              RB_METADATA_FIELD_TRACK_GAIN,
                              &val)) {
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_TRACK_GAIN, &val);
 		g_value_unset (&val);
 	}
@@ -1142,7 +1135,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 	if (rb_metadata_get (metadata,
 			     RB_METADATA_FIELD_TRACK_PEAK,
 			     &val)) {
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_TRACK_PEAK, &val);
 		g_value_unset (&val);
 	}
@@ -1151,7 +1144,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 	if (rb_metadata_get (metadata,
 			     RB_METADATA_FIELD_ALBUM_GAIN,
 			     &val)) {
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_ALBUM_GAIN, &val);
 		g_value_unset (&val);
 	}
@@ -1160,7 +1153,7 @@ set_props_from_metadata (RhythmDB *db, RhythmDBEntry *entry,
 	if (rb_metadata_get (metadata,
 			     RB_METADATA_FIELD_ALBUM_PEAK,
 			     &val)) {
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, TRUE,
 					     RHYTHMDB_PROP_ALBUM_PEAK, &val);
 		g_value_unset (&val);
 	}
@@ -1249,14 +1242,14 @@ rhythmdb_process_stat_event (RhythmDB *db, struct RhythmDBEvent *event)
 			g_get_current_time (&time);
 			g_value_init (&val, G_TYPE_ULONG);
 			g_value_set_ulong (&val, time.tv_sec);
-			rhythmdb_entry_set_internal (db, entry, 
+			rhythmdb_entry_set_internal (db, entry, TRUE,
 						     RHYTHMDB_PROP_LAST_SEEN,
 						     &val);
 			/* Old rhythmdb.xml files won't have a value for
 			 * FIRST_SEEN, so set it here
 			 */
 			if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_FIRST_SEEN) == 0) {
-				rhythmdb_entry_set_internal (db, entry, 
+				rhythmdb_entry_set_internal (db, entry, TRUE,
 							     RHYTHMDB_PROP_FIRST_SEEN,
 							     &val);
 			}
@@ -1305,6 +1298,7 @@ rhythmdb_process_metadata_load (RhythmDB *db, struct RhythmDBEvent *event)
 	RhythmDBEntry *entry;
 	GValue value = {0,};
 	const char *mime;
+	GTimeVal time;
 
 	if (event->error) {
 		struct RhythmDBLoadErrorData *data;
@@ -1335,8 +1329,11 @@ rhythmdb_process_metadata_load (RhythmDB *db, struct RhythmDBEvent *event)
 		return TRUE;
 	}
 
+	g_get_current_time (&time);
+
 	entry = rhythmdb_entry_lookup_by_location (db, event->real_uri);
 	if (!entry) {
+
 		entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, event->real_uri);
 		if (entry == NULL) {
 			rb_debug ("entry already exists");
@@ -1346,26 +1343,31 @@ rhythmdb_process_metadata_load (RhythmDB *db, struct RhythmDBEvent *event)
 		/* initialize the last played date to 0=never */
 		g_value_init (&value, G_TYPE_ULONG);
 		g_value_set_ulong (&value, 0);
-		rhythmdb_entry_set_internal (db, entry, 
-					     RHYTHMDB_PROP_LAST_PLAYED, &value);
+		rhythmdb_entry_set_uninserted (db, entry,
+					       RHYTHMDB_PROP_LAST_PLAYED, &value);
 		g_value_unset (&value);
 
 		/* initialize the rating */
 		g_value_init (&value, G_TYPE_DOUBLE);
 		g_value_set_double (&value, 2.5);
-		rhythmdb_entry_set_internal (db, entry, RHYTHMDB_PROP_RATING, &value);
+		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_RATING, &value);
 		g_value_unset (&value);
 
-		/* initialize auto rating */
-		g_value_init (&value, G_TYPE_BOOLEAN);
-		g_value_set_boolean (&value, TRUE);
-		rhythmdb_entry_set_internal (db, entry, 
-					     RHYTHMDB_PROP_AUTO_RATE, &value);
+	        /* first seen */
+		g_value_init (&value, G_TYPE_ULONG);
+		g_value_set_ulong (&value, time.tv_sec);
+		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_FIRST_SEEN, &value);
 		g_value_unset (&value);
 	}
 
 	set_props_from_metadata (db, entry, event->vfsinfo, event->metadata);
 
+	/* we've seen this entry */
+	g_value_init (&value, G_TYPE_ULONG);
+	g_value_set_ulong (&value, time.tv_sec);
+	rhythmdb_entry_set_internal (db, entry, TRUE, RHYTHMDB_PROP_LAST_SEEN, &value);
+	g_value_unset (&value);
+
 	/* Remember the mount point of the volume the song is on */
 	rhythmdb_entry_set_mount_point (db, entry, event->real_uri);
 
@@ -1382,13 +1384,14 @@ rhythmdb_process_queued_entry_set_event (RhythmDB *db,
 					 struct RhythmDBEvent *event)
 {
 	rhythmdb_entry_set_internal (db, event->entry, 
-			    event->change.prop, 
-			    &event->change.new);
+				     event->signal_change,
+				     event->change.prop, 
+				     &event->change.new);
 	/* Don't run rhythmdb_commit right now in case there 
 	 * we can run a single commit for several queued 
 	 * entry_set
 	 */
-	if (db->priv->commit_timeout_id) {
+	if (!db->priv->commit_timeout_id) {
 		db->priv->commit_timeout_id = g_timeout_add (100, (GSourceFunc)timeout_rhythmdb_commit, db);
 	}
 }
@@ -1567,18 +1570,11 @@ queue_stat_uri (const char *uri, RhythmDB *db)
 static gpointer
 add_thread_main (struct RhythmDBAddThreadData *data)
 {
-	char  *realuri;
 	struct RhythmDBEvent *result;
 
-	realuri = rb_uri_resolve_symlink (data->uri);
+	rb_uri_handle_recursively (data->uri, (GFunc) queue_stat_uri,
+				   &data->db->priv->exiting, data->db);
 
-	if (rb_uri_is_directory (realuri))
-		rb_uri_handle_recursively (data->uri, (GFunc) queue_stat_uri,
-					   &data->db->priv->exiting, data->db);
-	else 
-		queue_stat_uri (realuri, data->db);
-	
-	g_free (realuri);
 	rb_debug ("exiting");
 	result = g_new0 (struct RhythmDBEvent, 1);
 	result->type = RHYTHMDB_EVENT_THREAD_EXITED;
@@ -1618,7 +1614,7 @@ error:
 	event->vfsinfo = NULL;
 }
 
-static void
+void
 rhythmdb_entry_get (RhythmDBEntry *entry, 
 		    RhythmDBPropType propid, GValue *val)
 {
@@ -1760,11 +1756,18 @@ action_thread_main (RhythmDB *db)
 void
 rhythmdb_add_uri (RhythmDB *db, const char *uri)
 {
-	struct RhythmDBAddThreadData *data = g_new0 (struct RhythmDBAddThreadData, 1);
-	data->db = db;
-	data->uri = g_strdup (uri);
+	char  *realuri = rb_uri_resolve_symlink (uri);
 
-	rhythmdb_thread_create (db, (GThreadFunc) add_thread_main, data);
+	if (rb_uri_is_directory (realuri)) {
+		struct RhythmDBAddThreadData *data = g_new0 (struct RhythmDBAddThreadData, 1);
+		data->db = db;
+		data->uri = realuri;
+
+		rhythmdb_thread_create (db, (GThreadFunc) add_thread_main, data);
+	} else {
+		queue_stat_uri (realuri, db);
+		g_free (realuri);
+	}
 }
 
 #if 0
@@ -1880,10 +1883,70 @@ rhythmdb_save (RhythmDB *db)
 	g_mutex_unlock (db->priv->saving_mutex);
 }
 
-/* FIXME: this has nothing to do with entry_sync_mirrored */
+static void
+threadsafe_entry_set (RhythmDB *db, RhythmDBEntry *entry,
+		      gboolean notify, guint propid, const GValue *value)
+{
+	if (!rhythmdb_get_readonly (db) && rb_is_main_thread ()) {
+		rhythmdb_entry_set_internal (db, entry, notify, propid, value);
+	} else {
+		struct RhythmDBEvent *result;
+
+		result = g_new0 (struct RhythmDBEvent, 1);
+		result->type = RHYTHMDB_EVENT_ENTRY_SET;
+
+		rb_debug ("queuing RHYTHMDB_ACTION_ENTRY_SET");
+
+		result->entry = entry;
+		result->change.prop = propid;
+		result->signal_change = notify;
+		g_value_init (&result->change.new, G_VALUE_TYPE (value));
+		g_value_copy (value, &result->change.new);
+		g_async_queue_push (db->priv->event_queue, result);
+	}
+}
+
+/*
+ * This function can be called by any code which wishes to change a
+ * song property and send a notification.  It may be called when the
+ * database is read-only; in this case the change will be queued for
+ * an unspecified time in the future.  The implication of this is that
+ * rhythmdb_entry_get (or direct access of the property) may not
+ * reflect the changes immediately.  However, if this property is
+ * exposed in the user interface, you should still make the change in
+ * the widget.  Then when the database returns to a writable state,
+ * your change will take effect in the database too, and a notification
+ * will be sent at that point.
+ *
+ * Note that you must call rhythmdb_commit at some point after invoking
+ * this function, and that even after the commit, your change may not
+ * have taken effect.
+ */
 void 
-rhythmdb_entry_sync (RhythmDB *db, RhythmDBEntry *entry,
-		     guint propid, GValue *value)
+rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry, 
+		    guint propid, const GValue *value)
+{
+	g_return_if_fail (entry->inserted == TRUE);
+	threadsafe_entry_set (db, entry, TRUE, propid, value);
+}
+
+/*
+ * This function is like rhythmdb_entry_set, except no notification
+ * of the change will be sent.  This is useful if you know no
+ * one could possibly be listening for the change.  In this case
+ * you do not need to call rhythmdb_commit.
+ */
+void 
+rhythmdb_entry_set_nonotify (RhythmDB *db, RhythmDBEntry *entry, 
+			     guint propid, const GValue *value)
+{
+	g_return_if_fail (!entry->inserted);
+	threadsafe_entry_set (db, entry, FALSE, propid, value);
+}
+
+static void
+record_entry_change (RhythmDB *db, RhythmDBEntry *entry,
+		     guint propid, const GValue *value)
 {
 	RhythmDBEntryChange *changedata;
 	GSList *changelist;
@@ -1907,22 +1970,29 @@ rhythmdb_entry_sync (RhythmDB *db, RhythmDBEntry *entry,
 	changelist = g_hash_table_lookup (db->priv->changed_entries, entry);
 	changelist = g_slist_append (changelist, changedata);
 	g_hash_table_insert (db->priv->changed_entries, entry, changelist);
-
-	rhythmdb_entry_set_internal (db, entry, propid, value);
 }
 
-/* This function must be called from the main thread when the database isn't
- * read-only
+/*
+ * This function should only be called for entries which have
+ * been created with rhythmdb_entry_new, but not yet committed
+ * to the database (i.e. before rhythmdb_commit).
  */
+void
+rhythmdb_entry_set_uninserted (RhythmDB *db, RhythmDBEntry *entry,
+			       guint propid, const GValue *value)
+{
+	g_return_if_fail (entry->inserted == FALSE);
+
+	rhythmdb_entry_set_internal (db, entry, FALSE, propid, value);
+}
+
 static void
 rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
-			     guint propid, GValue *value)
+			     gboolean notify_if_inserted,
+			     guint propid, const GValue *value)
 {
 	RhythmDBClass *klass = RHYTHMDB_GET_CLASS (db);
 
-	g_assert (rb_is_main_thread ());
-	g_assert (rhythmdb_get_readonly (db) == FALSE);
-
 #ifndef G_DISABLE_ASSERT	
 	switch (G_VALUE_TYPE (value))
 	{
@@ -1940,6 +2010,10 @@ rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
 	}
 #endif
 
+	if (entry->inserted && notify_if_inserted) {
+		record_entry_change (db, entry, propid, value);
+	}
+
 	klass->impl_entry_set (db, entry, propid, value);
 
 	switch (propid)
@@ -2019,9 +2093,6 @@ rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
 	case RHYTHMDB_PROP_RATING:
 		entry->rating = g_value_get_double (value);
 		break;
-	case RHYTHMDB_PROP_AUTO_RATE:
-		entry->auto_rate = g_value_get_boolean (value);
-		break;
 	case RHYTHMDB_PROP_PLAY_COUNT:
 		entry->play_count = g_value_get_ulong (value);
 		break;
@@ -2041,28 +2112,6 @@ rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
 	db->priv->dirty = TRUE;
 }
 
-void 
-rhythmdb_entry_set (RhythmDB *db, RhythmDBEntry *entry, 
-		    guint propid, GValue *value)
-{
-	if (!rhythmdb_get_readonly (db) && rb_is_main_thread ()) {
-		rhythmdb_entry_set_internal (db, entry, propid, value);
-	} else {
-		struct RhythmDBEvent *result;
-
-		result = g_new0 (struct RhythmDBEvent, 1);
-		result->type = RHYTHMDB_EVENT_ENTRY_SET;
-
-		rb_debug ("queuing RHYTHMDB_ACTION_ENTRY_SET");
-
-		result->entry = entry;
-		result->change.prop = propid;
-		g_value_init (&result->change.new, G_VALUE_TYPE (value));
-		g_value_copy (value, &result->change.new);
-		g_async_queue_push (db->priv->event_queue, result);
-	}	
-}
-
 void
 rhythmdb_entry_sync_mirrored (RhythmDB *db, RhythmDBEntry *entry, guint propid)
 {
@@ -2077,12 +2126,23 @@ rhythmdb_entry_sync_mirrored (RhythmDB *db, RhythmDBEntry *entry, guint propid)
 			entry->last_played_str = rb_refstring_new_full (_("Never"), FALSE);
 		else {
 			val = eel_strdup_strftime (_("%Y-%m-%d %H:%M"),
-						   localtime (&entry->last_played));
+						   localtime ((glong*)&entry->last_played));
 			entry->last_played_str = rb_refstring_new_full (val, FALSE);
 			g_free (val);
 		}
 		break;
 	}
+	case RHYTHMDB_PROP_FIRST_SEEN:
+	{
+		if (entry->first_seen_str)
+			rb_refstring_unref (entry->first_seen_str);
+
+		val = eel_strdup_strftime (_("%Y-%m-%d %H:%M"),
+					   localtime ((glong*)&entry->first_seen));
+		entry->first_seen_str = rb_refstring_new_full (val, FALSE);
+		g_free (val);
+		break;
+	}
 	default:
 		break;
 	}
@@ -2164,6 +2224,8 @@ rhythmdb_query_parse_valist (RhythmDB *db, va_list args)
 		case RHYTHMDB_QUERY_PROP_NOT_LIKE:
 		case RHYTHMDB_QUERY_PROP_GREATER:
 		case RHYTHMDB_QUERY_PROP_LESS:
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN:
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
 			data->propid = va_arg (args, guint);
 			data->val = g_new0 (GValue, 1);
 			g_value_init (data->val, rhythmdb_get_property_type (db, data->propid));
@@ -2234,6 +2296,8 @@ rhythmdb_query_free (GPtrArray *query)
 		case RHYTHMDB_QUERY_PROP_NOT_LIKE:
 		case RHYTHMDB_QUERY_PROP_GREATER:
 		case RHYTHMDB_QUERY_PROP_LESS:
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN:
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
 			g_value_unset (data->val);
 			g_free (data->val);
 			break;
@@ -2392,6 +2456,16 @@ rhythmdb_query_serialize (RhythmDB *db, GPtrArray *query,
 			xmlSetProp (subnode, RB_PARSE_PROP, rhythmdb_nice_elt_name_from_propid (db, data->propid));
 			write_encoded_gvalue (subnode, data->val);
 			break;
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN:
+			subnode = xmlNewChild (node, NULL, RB_PARSE_CURRENT_TIME_WITHIN, NULL);
+			xmlSetProp (subnode, RB_PARSE_PROP, rhythmdb_nice_elt_name_from_propid (db, data->propid));
+			write_encoded_gvalue (subnode, data->val);
+			break;
+		case RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN:
+			subnode = xmlNewChild (node, NULL, RB_PARSE_CURRENT_TIME_NOT_WITHIN, NULL);
+			xmlSetProp (subnode, RB_PARSE_PROP, rhythmdb_nice_elt_name_from_propid (db, data->propid));
+			write_encoded_gvalue (subnode, data->val);
+			break;
 		}		
 	}
 }
@@ -2432,6 +2506,10 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 			data->type = RHYTHMDB_QUERY_PROP_GREATER;
 		} else if (!xmlStrcmp (child->name, RB_PARSE_LESS)) {
 			data->type = RHYTHMDB_QUERY_PROP_LESS;
+		} else if (!xmlStrcmp (child->name, RB_PARSE_CURRENT_TIME_WITHIN)) {
+			data->type = RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN;
+		} else if (!xmlStrcmp (child->name, RB_PARSE_CURRENT_TIME_NOT_WITHIN)) {
+			data->type = RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN;
 		} else
  			g_assert_not_reached ();
 
@@ -2439,7 +2517,9 @@ rhythmdb_query_deserialize (RhythmDB *db, xmlNodePtr parent)
 		    || !xmlStrcmp (child->name, RB_PARSE_NOT_LIKE)
 		    || !xmlStrcmp (child->name, RB_PARSE_EQUALS)
 		    || !xmlStrcmp (child->name, RB_PARSE_GREATER)
-		    || !xmlStrcmp (child->name, RB_PARSE_LESS)) {
+		    || !xmlStrcmp (child->name, RB_PARSE_LESS)
+		    || !xmlStrcmp (child->name, RB_PARSE_CURRENT_TIME_WITHIN)
+		    || !xmlStrcmp (child->name, RB_PARSE_CURRENT_TIME_NOT_WITHIN)) {
 			xmlChar *propstr = xmlGetProp (child, RB_PARSE_PROP);
 			gint propid = rhythmdb_propid_from_nice_elt_name (db, propstr);
 			g_free (propstr);
@@ -2488,6 +2568,8 @@ rhythmdb_query_internal (struct RhythmDBQueryThreadData *data)
 	struct RhythmDBEvent *result;
 	RhythmDBClass *klass = RHYTHMDB_GET_CLASS (data->db);
 
+	rhythmdb_query_preprocess (data->query);
+
 	rb_debug ("doing query");
 
 	klass->impl_do_full_query (data->db, data->query,
@@ -2628,6 +2710,8 @@ rhythmdb_query_get_type (void)
 			ENUM_ENTRY (RHYTHMDB_QUERY_PROP_NOT_LIKE, "Inverted fuzzy property matching"),
 			ENUM_ENTRY (RHYTHMDB_QUERY_PROP_GREATER, "True if property1 >= property2"),
 			ENUM_ENTRY (RHYTHMDB_QUERY_PROP_LESS, "True if property1 <= property2"),
+			ENUM_ENTRY (RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN, "True if property1 is within property2 of the current time"),
+			ENUM_ENTRY (RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN, "True if property1 is not within property2 of the current time"),
 			{ 0, 0, 0 }
 		};
 
@@ -2669,7 +2753,6 @@ rhythmdb_prop_get_type (void)
 			ENUM_ENTRY (RHYTHMDB_PROP_FIRST_SEEN, "Time the song was added to the library (gulong) [first-seen]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_LAST_SEEN, "Last time the song was available (gulong) [last-seen]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_RATING, "Rating (gdouble) [rating]"),
-			ENUM_ENTRY (RHYTHMDB_PROP_AUTO_RATE, "Whether to auto-rate song (gboolean) [auto-rate]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_PLAY_COUNT, "Play Count (gulong) [play-count]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED, "Last Played (gulong) [last-played]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_BITRATE, "Bitrate (gulong) [bitrate]"),
@@ -2690,6 +2773,7 @@ rhythmdb_prop_get_type (void)
 			ENUM_ENTRY (RHYTHMDB_PROP_LAST_PLAYED_STR, "Last Played (gchararray) [last-played-str]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_PLAYBACK_ERROR, "Playback error string (gchararray) [playback-error]"),
 			ENUM_ENTRY (RHYTHMDB_PROP_HIDDEN, "Visibility (gboolean) [visibility]"),
+			ENUM_ENTRY (RHYTHMDB_PROP_FIRST_SEEN_STR, "Time Added to Library (gchararray) [first-seen-str]"),
 			{ 0, 0, 0 }
 		};
 		g_assert ((sizeof (values) / sizeof (values[0]) - 1) == RHYTHMDB_NUM_PROPERTIES);
@@ -2727,9 +2811,9 @@ char *
 rhythmdb_compute_status_normal (gint n_songs, glong duration, GnomeVFSFileSize size)
 {
 	long days, hours, minutes, seconds;
-	char *songcount;
-	char *time;
-	char *size_str;
+	char *songcount = NULL;
+	char *time = NULL;
+	char *size_str = NULL;
 	char *ret;
 	const char *minutefmt;
 	const char *hourfmt;	
@@ -2788,7 +2872,17 @@ rhythmdb_compute_status_normal (gint n_songs, glong duration, GnomeVFSFileSize s
 	}
 
 	size_str = gnome_vfs_format_file_size_for_display (size);
-	ret = g_strdup_printf ("%s, %s, %s", songcount, time, size_str);
+
+	if (size > 0 && duration > 0) {
+		ret = g_strdup_printf ("%s, %s, %s", songcount, time, size_str);
+	} else if (duration > 0) {
+		ret = g_strdup_printf ("%s, %s", songcount, time);
+	} else if (size > 0) {
+		ret = g_strdup_printf ("%s, %s", songcount, size_str);
+	} else {
+		ret = g_strdup (songcount);
+	}
+
 	g_free (songcount);
 	g_free (time);
 	g_free (size_str);
@@ -2864,7 +2958,7 @@ entry_volume_mounted_or_unmounted (RhythmDBEntry *entry,
 		g_get_current_time (&time);
 		g_value_init (&val, G_TYPE_ULONG);
 		g_value_set_ulong (&val, time.tv_sec);
-		rhythmdb_entry_set_internal (ctxt->db, entry, 
+		rhythmdb_entry_set_internal (ctxt->db, entry, FALSE,
 					     RHYTHMDB_PROP_LAST_SEEN, &val);
 		g_value_unset (&val);
 
@@ -2918,7 +3012,7 @@ rhythmdb_entry_set_mount_point (RhythmDB *db, RhythmDBEntry *entry,
 	if (mount_point != NULL) {
 		g_value_init (&value, G_TYPE_STRING);
 		g_value_set_string_take_ownership (&value, mount_point);
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, FALSE,
 					     RHYTHMDB_PROP_MOUNTPOINT, 
 					     &value);
 		g_value_unset (&value);
@@ -2944,7 +3038,7 @@ rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry,
 		
 		g_value_init (&new_val, G_TYPE_BOOLEAN);
 		g_value_set_boolean (&new_val, !visible);
-		rhythmdb_entry_set_internal (db, entry, 
+		rhythmdb_entry_set_internal (db, entry, FALSE,
 					     RHYTHMDB_PROP_HIDDEN, &new_val);
 		
 		g_signal_emit (G_OBJECT (db), rhythmdb_signals[ENTRY_CHANGED], 
@@ -2955,3 +3049,31 @@ rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry,
 	}
 	g_value_unset (&old_val);
 }
+
+void
+rhythmdb_query_preprocess (GPtrArray *query)
+{
+	int i;	
+
+	for (i = 0; i < query->len; i++) {
+		RhythmDBQueryData *data = g_ptr_array_index (query, i);
+		
+		if (data->subquery) {
+			rhythmdb_query_preprocess (data->subquery);
+		} else switch (data->propid) {
+			case RHYTHMDB_PROP_TITLE_FOLDED:
+			case RHYTHMDB_PROP_GENRE_FOLDED:
+			case RHYTHMDB_PROP_ARTIST_FOLDED:
+			case RHYTHMDB_PROP_ALBUM_FOLDED:
+			{
+				/* as we are matching against a folded property, the string needs to also be folded */
+				const char *orig = g_value_get_string (data->val);
+				char *folded = g_utf8_casefold (orig, -1);
+
+				g_value_reset (data->val);
+				g_value_take_string (data->val, folded);
+				break;
+			}
+		}
+	}
+}
