@@ -1,7 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: Implementation of Rhythmbox playlist management object
  *
- *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,19 +23,23 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-i18n.h>
-#include <bonobo/bonobo-ui-util.h>
 #include <libxml/tree.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
+#include <stdio.h>      /* rename() */
+#include <unistd.h>     /* unlink() */
 
 #include "rb-playlist-manager.h"
 #include "rb-playlist-source.h"
+#if defined(WITH_CD_BURNER_SUPPORT)
+#include "rb-recorder.h"
+#endif
 #include "rb-sourcelist.h"
+#include "rb-sourcelist-model.h"
 #include "rb-query-creator.h"
-#include "rb-playlist.h"
+#include "totem-pl-parser.h"
 
-#include "rb-bonobo-helpers.h"
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
@@ -42,6 +47,9 @@
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
 
+#define RB_PLAYLIST_MGR_VERSION (xmlChar *) "1.0"
+#define RB_PLAYLIST_MGR_PL (xmlChar *) "rhythmdb-playlists"
+
 static void rb_playlist_manager_class_init (RBPlaylistManagerClass *klass);
 static void rb_playlist_manager_init (RBPlaylistManager *mgr);
 static void rb_playlist_manager_finalize (GObject *object);
@@ -53,42 +61,42 @@ static void rb_playlist_manager_get_property (GObject *object,
 					      guint prop_id,
 					      GValue *value,
 					      GParamSpec *pspec);
-static void rb_playlist_manager_cmd_load_playlist (BonoboUIComponent *component,
-						   RBPlaylistManager *mgr,
-						   const char *verbname);
-static void rb_playlist_manager_cmd_save_playlist (BonoboUIComponent *component,
-						   RBPlaylistManager *mgr,
-						   const char *verbname);
-static void rb_playlist_manager_cmd_new_playlist (BonoboUIComponent *component,
-						  RBPlaylistManager *mgr,
-						  const char *verbname);
-static void rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *component,
-							    RBPlaylistManager *mgr,
-							    const char *verbname);
-static void rb_playlist_manager_cmd_delete_playlist (BonoboUIComponent *component,
-						     RBPlaylistManager *mgr,
-						     const char *verbname);
-static void rb_playlist_manager_cmd_edit_automatic_playlist (BonoboUIComponent *component,
-							     RBPlaylistManager *mgr,
-							     const char *verbname);
-static void handle_playlist_entry_into_playlist_cb (RBPlaylist *playlist, const char *uri, const char *title,
-						    const char *genre, RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_load_playlist (GtkAction *action,
+						   RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_save_playlist (GtkAction *action,
+						   RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_burn_playlist (GtkAction *action,
+						   RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_new_playlist (GtkAction *action,
+						  RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
+							    RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_rename_playlist (GtkAction *action,
+						     RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_delete_playlist (GtkAction *action,
+						     RBPlaylistManager *mgr);
+static void rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
+							     RBPlaylistManager *mgr);
 static gboolean reap_dead_playlist_threads (RBPlaylistManager *mgr);
+static void rb_playlist_manager_playlist_entries_changed (RBEntryView *entry_view,
+							   RhythmDBEntry *entry,
+							   RBPlaylistManager *mgr);
 
 struct RBPlaylistManagerPrivate
 {
 	RhythmDB *db;
+	RBShell *shell;
 	RBSource *selected_source;
 
 	RBSourceList *sourcelist;
 
-	BonoboUIComponent *component;
+	GtkActionGroup *actiongroup;
+	GtkUIManager *uimanager;
+
 	RBLibrarySource *libsource;
 	RBIRadioSource *iradio_source;
 	GtkWindow *window;
 
-	GList *playlists;
-
 	guint playlist_serial;
 
 	RBPlaylistSource *loading_playlist;
@@ -110,10 +118,8 @@ struct RBPlaylistManagerPrivate
 enum
 {
 	PROP_0,
-	PROP_WINDOW,
-	PROP_COMPONENT,
+	PROP_SHELL,
 	PROP_SOURCE,
-	PROP_DB,
 	PROP_SOURCELIST,
 	PROP_LIBRARY_SOURCE,
 	PROP_IRADIO_SOURCE,
@@ -122,6 +128,7 @@ enum
 enum
 {
 	PLAYLIST_ADDED,
+	PLAYLIST_CREATED,
 	PLAYLIST_LOAD_START,
 	PLAYLIST_LOAD_FINISH,
 	LAST_SIGNAL,
@@ -129,20 +136,37 @@ enum
 
 static guint rb_playlist_manager_signals[LAST_SIGNAL] = { 0 };
 
-#define CMD_PATH_PLAYLIST_DELETE   "/commands/FileDeletePlaylist"
-#define CMD_PATH_PLAYLIST_SAVE   "/commands/SavePlaylist"
-
-static BonoboUIVerb rb_playlist_manager_verbs[] =
+static GtkActionEntry rb_playlist_manager_actions [] =
 {
- 	BONOBO_UI_VERB ("NewPlaylist",  (BonoboUIVerbFn) rb_playlist_manager_cmd_new_playlist),
- 	BONOBO_UI_VERB ("NewAutomaticPlaylist",  (BonoboUIVerbFn) rb_playlist_manager_cmd_new_automatic_playlist),
-	BONOBO_UI_VERB ("LoadPlaylist", (BonoboUIVerbFn) rb_playlist_manager_cmd_load_playlist),
-	BONOBO_UI_VERB ("SavePlaylist", (BonoboUIVerbFn) rb_playlist_manager_cmd_save_playlist),
-	BONOBO_UI_VERB ("FileDeletePlaylist",(BonoboUIVerbFn) rb_playlist_manager_cmd_delete_playlist),
-	BONOBO_UI_VERB ("DeletePlaylist",  (BonoboUIVerbFn) rb_playlist_manager_cmd_delete_playlist),
- 	BONOBO_UI_VERB ("EditAutomaticPlaylist",  (BonoboUIVerbFn) rb_playlist_manager_cmd_edit_automatic_playlist),
-	BONOBO_UI_VERB_END
+	/* Submenu of Music */
+	{ "Playlist", NULL, N_("_Playlist") },
+
+	{ "MusicPlaylistNewPlaylist", "gtk-media-playlist", N_("_New Playlist..."), "<control>N",
+	  N_("Create a new playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_new_playlist) },
+	{ "MusicPlaylistNewAutomaticPlaylist", "gtk-media-automatic-playlist", N_("New _Automatic Playlist..."), NULL,
+	  N_("Create a new automatically updating playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_new_automatic_playlist) },
+	{ "MusicPlaylistLoadPlaylist", NULL, N_("_Load from File..."), NULL,
+	  N_("Choose a playlist to be loaded"),
+	  G_CALLBACK (rb_playlist_manager_cmd_load_playlist) },
+	{ "MusicPlaylistSavePlaylist", GTK_STOCK_SAVE_AS, N_("_Save to File..."), NULL,
+	  N_("Save a playlist to a file"),
+	  G_CALLBACK (rb_playlist_manager_cmd_save_playlist) },
+	{ "MusicPlaylistBurnPlaylist", GTK_STOCK_CDROM, N_("_Create Audio CD..."), NULL,
+	  N_("Create an audio CD from playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_burn_playlist) },
+	{ "MusicPlaylistRenamePlaylist", NULL, N_("_Rename..."), NULL,
+	  N_("Rename playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_rename_playlist) },
+	{ "MusicPlaylistDeletePlaylist", GTK_STOCK_REMOVE, N_("_Delete"), NULL,
+	  N_("Delete playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_delete_playlist) },
+	{ "EditAutomaticPlaylist", GTK_STOCK_PROPERTIES, N_("_Edit"), NULL,
+	  N_("Change this automatic playlist"),
+	  G_CALLBACK (rb_playlist_manager_cmd_edit_automatic_playlist) },
 };
+static guint rb_playlist_manager_n_actions = G_N_ELEMENTS (rb_playlist_manager_actions);
 
 static GObjectClass *parent_class = NULL;
 
@@ -193,29 +217,16 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 							      "RBSource object",
 							      RB_TYPE_SOURCE,
 							      G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
-					 PROP_WINDOW,
-					 g_param_spec_object ("window",
-							      "window",
-							      "toplevel window",
-							      GTK_TYPE_WINDOW,
+					 PROP_SHELL,
+					 g_param_spec_object ("shell",
+							      "RBShell",
+							      "RBShell object",
+							      RB_TYPE_SHELL,
 							      G_PARAM_READWRITE));
 
-	g_object_class_install_property (object_class,
-					 PROP_COMPONENT,
-					 g_param_spec_object ("component",
-							      "BonoboUIComponent",
-							      "BonoboUIComponent object",
-							      BONOBO_TYPE_UI_COMPONENT,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property (object_class,
-					 PROP_DB,
-					 g_param_spec_object ("db",
-							      "RhythmDB",
-							      "RhythmDB database",
-							      RHYTHMDB_TYPE,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_LIBRARY_SOURCE,
 					 g_param_spec_object ("library_source",
@@ -247,6 +258,16 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
+	
+	rb_playlist_manager_signals[PLAYLIST_CREATED] =
+		g_signal_new ("playlist_created",
+			      RB_TYPE_PLAYLIST_MANAGER,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlaylistManagerClass, playlist_created),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, G_TYPE_OBJECT);
 
 	rb_playlist_manager_signals[PLAYLIST_LOAD_START] =
 		g_signal_new ("load_start",
@@ -329,13 +350,40 @@ rb_playlist_manager_finalize (GObject *object)
 	g_mutex_free (mgr->priv->saving_mutex);
 	g_cond_free (mgr->priv->saving_condition);
 
-	g_list_free (mgr->priv->playlists);
-
 	g_free (mgr->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr, 
+				   GtkUIManager *uimanager)
+{
+
+	if (mgr->priv->uimanager != NULL) {
+		if (mgr->priv->actiongroup != NULL) {
+			gtk_ui_manager_remove_action_group (mgr->priv->uimanager,
+							    mgr->priv->actiongroup);
+		}
+		g_object_unref (G_OBJECT (mgr->priv->uimanager));
+		mgr->priv->uimanager = NULL;
+	}
+
+	mgr->priv->uimanager = uimanager;
+
+	if (mgr->priv->actiongroup == NULL) {
+		mgr->priv->actiongroup = gtk_action_group_new ("PlaylistActions");
+		gtk_action_group_add_actions (mgr->priv->actiongroup,
+					      rb_playlist_manager_actions,
+					      rb_playlist_manager_n_actions,
+					      mgr);
+	}
+	
+	gtk_ui_manager_insert_action_group (mgr->priv->uimanager,
+					    mgr->priv->actiongroup,
+					    0);
+}
+
 static void
 rb_playlist_manager_set_property (GObject *object,
 				  guint prop_id,
@@ -349,28 +397,59 @@ rb_playlist_manager_set_property (GObject *object,
 	case PROP_SOURCE:
 	{
 		gboolean playlist_active;
+		GtkAction *action;
+
+		if (mgr->priv->selected_source != NULL)
+			g_signal_handlers_disconnect_by_func (G_OBJECT (rb_source_get_entry_view (mgr->priv->selected_source)),
+							      G_CALLBACK (rb_playlist_manager_playlist_entries_changed),
+							      mgr);
 
 		mgr->priv->selected_source = g_value_get_object (value);
+		playlist_active = RB_IS_PLAYLIST_SOURCE (mgr->priv->selected_source);
 
-		playlist_active = g_list_find (mgr->priv->playlists,
-					       mgr->priv->selected_source) != NULL;
-		rb_bonobo_set_sensitive (mgr->priv->component, CMD_PATH_PLAYLIST_SAVE,
-					 playlist_active);
-		rb_bonobo_set_sensitive (mgr->priv->component, CMD_PATH_PLAYLIST_DELETE,
-					 playlist_active);
+		action = gtk_action_group_get_action (mgr->priv->actiongroup,
+						      "MusicPlaylistSavePlaylist");
+		g_object_set (G_OBJECT (action), "sensitive", playlist_active, NULL);
+		action = gtk_action_group_get_action (mgr->priv->actiongroup,
+						      "MusicPlaylistDeletePlaylist");
+		g_object_set (G_OBJECT (action), "sensitive", playlist_active, NULL);
+#if defined(WITH_CD_BURNER_SUPPORT)
+		{
+			gboolean recorder_active;
+			int num_tracks = rb_entry_view_get_num_entries (rb_source_get_entry_view (mgr->priv->selected_source));
+
+			recorder_active = playlist_active && rb_recorder_enabled ()
+					&& (num_tracks > 0);
+			action = gtk_action_group_get_action (mgr->priv->actiongroup, "MusicPlaylistBurnPlaylist");
+			g_object_set (G_OBJECT (action), "sensitive", recorder_active, NULL);
+			
+			if (playlist_active && rb_recorder_enabled ()) {
+				/* monitor for changes, to enable/disable the burn menu item */
+				g_signal_connect_object (G_OBJECT (rb_source_get_entry_view (mgr->priv->selected_source)),
+							 "entry-added", G_CALLBACK (rb_playlist_manager_playlist_entries_changed), mgr, 0);
+				g_signal_connect_object (G_OBJECT (rb_source_get_entry_view (mgr->priv->selected_source)),
+							 "entry-deleted", G_CALLBACK (rb_playlist_manager_playlist_entries_changed), mgr, 0);
+			}
+		}
+#endif
 		break;
 	}
-	case PROP_COMPONENT:
-		mgr->priv->component = g_value_get_object (value);
-		bonobo_ui_component_add_verb_list_with_data (mgr->priv->component,
-							     rb_playlist_manager_verbs,
-							     mgr);
-		break;
-	case PROP_DB:
-		mgr->priv->db = g_value_get_object (value);
+	case PROP_SHELL:
+	{
+		GtkUIManager *uimanager;
+		mgr->priv->shell = g_value_get_object (value);
+		g_object_get (G_OBJECT (mgr->priv->shell), 
+			      "ui-manager", &uimanager, 
+			      "db", &mgr->priv->db,
+			      NULL);
+		rb_playlist_manager_set_uimanager (mgr, uimanager);
+			
+			      
 		break;
+	}
 	case PROP_SOURCELIST:
 		mgr->priv->sourcelist = g_value_get_object (value);
+		mgr->priv->window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (mgr->priv->sourcelist)));
 		break;
 	case PROP_LIBRARY_SOURCE:
 		mgr->priv->libsource = g_value_get_object (value);
@@ -378,9 +457,6 @@ rb_playlist_manager_set_property (GObject *object,
 	case PROP_IRADIO_SOURCE:
 		mgr->priv->iradio_source = g_value_get_object (value);
 		break;
-	case PROP_WINDOW:
-		mgr->priv->window = g_value_get_object (value);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -400,11 +476,8 @@ rb_playlist_manager_get_property (GObject *object,
 	case PROP_SOURCE:
 		g_value_set_object (value, mgr->priv->selected_source);
 		break;
-	case PROP_COMPONENT:
-		g_value_set_object (value, mgr->priv->component);
-		break;
-	case PROP_DB:
-		g_value_set_object (value, mgr->priv->db);
+	case PROP_SHELL:
+		g_value_set_object (value, mgr->priv->shell);
 		break;
 	case PROP_SOURCELIST:
 		g_value_set_object (value, mgr->priv->sourcelist);
@@ -415,9 +488,6 @@ rb_playlist_manager_get_property (GObject *object,
 	case PROP_IRADIO_SOURCE:
 		g_value_set_object (value, mgr->priv->iradio_source);
 		break;
-	case PROP_WINDOW:
-		g_value_set_object (value, mgr->priv->window);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -436,27 +506,82 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr, RBSource *source)
 }
 
 RBPlaylistManager *
-rb_playlist_manager_new (BonoboUIComponent *component, GtkWindow *window,
-			 RhythmDB *db, RBSourceList *sourcelist,
+rb_playlist_manager_new (RBShell *shell,
+			 RBSourceList *sourcelist,
 			 RBLibrarySource *libsource,
 			 RBIRadioSource *iradio_source)
 {
-	RBPlaylistManager *mgr = g_object_new (RB_TYPE_PLAYLIST_MANAGER,
-					       "component", component,
-					       "window", window,
-					       "db", db,
-					       "sourcelist", sourcelist,
-					       "library_source", libsource,
-					       "iradio_source", iradio_source,
-					       NULL);
-
-	g_return_val_if_fail (mgr->priv != NULL, NULL);
-
-	return mgr;
+	return g_object_new (RB_TYPE_PLAYLIST_MANAGER,
+			     "shell", shell,
+			     "sourcelist", sourcelist,
+			     "library_source", libsource,
+			     "iradio_source", iradio_source,
+			     NULL);
 }
 
-const char *
-rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri)
+GQuark
+rb_playlist_manager_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark)
+		quark = g_quark_from_static_string ("rb_playlist_manager_error");
+
+	return quark;
+}
+
+static void
+handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri_maybe,
+			  const char *title,
+			  const char *genre, RBPlaylistManager *mgr)
+{
+	char *uri = rb_canonicalise_uri (uri_maybe);
+	gint entry_type;
+
+	g_return_if_fail (uri != NULL);
+
+	entry_type = rb_shell_guess_type_for_uri (mgr->priv->shell, uri);
+	if (entry_type < 0) {
+		return;
+	}
+
+	rb_shell_add_uri (mgr->priv->shell,
+			  entry_type,
+			  uri,
+			  title,
+			  genre,
+			  NULL);
+
+	if (entry_type == RHYTHMDB_ENTRY_TYPE_SONG) {
+		if (!mgr->priv->loading_playlist) {
+			mgr->priv->loading_playlist =
+				RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
+		}
+		rb_playlist_source_add_location (mgr->priv->loading_playlist, uri);
+	}
+
+	g_free (uri);
+}
+
+static void
+playlist_load_start_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
+{
+	rb_debug ("loading new playlist %s", title);
+
+	if (!mgr->priv->loading_playlist) {
+		mgr->priv->loading_playlist =
+			RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
+	}
+}
+
+static void
+playlist_load_end_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
+{
+	rb_debug ("finished loading playlist %s", title);
+	mgr->priv->loading_playlist = NULL;
+}
+
+gboolean
+rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError **error)
 {
 	rb_debug ("loading playlist from %s", uri);
 
@@ -466,36 +591,40 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri)
 	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_LOAD_START], 0);
 
 	{
-		RBPlaylist *parser = rb_playlist_new ();
+		TotemPlParser *parser = totem_pl_parser_new ();
 
 		g_signal_connect_object (G_OBJECT (parser), "entry",
-					 G_CALLBACK (handle_playlist_entry_into_playlist_cb),
+					 G_CALLBACK (handle_playlist_entry_cb),
 					 mgr, 0);
 
-		if (!rb_playlist_parse (parser, uri))
-			rb_error_dialog (_("Couldn't parse playlist"));
+		g_signal_connect_object (G_OBJECT (parser), "playlist-start",
+					 G_CALLBACK (playlist_load_start_cb),
+					 mgr, 0);
+
+		g_signal_connect_object (G_OBJECT (parser), "playlist-end",
+					 G_CALLBACK (playlist_load_end_cb),
+					 mgr, 0);
+		
+		if (totem_pl_parser_parse (parser, uri, FALSE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+			g_set_error (error,
+				     RB_PLAYLIST_MANAGER_ERROR,
+				     RB_PLAYLIST_MANAGER_ERROR_PARSE,
+				     "%s",
+				     _("The playlist file may be in an unknown format or corrupted."));
+			return FALSE;
+		}
 		mgr->priv->loading_playlist = NULL;
 
 		g_object_unref (G_OBJECT (parser));
 	}
 
 	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_LOAD_FINISH], 0);
-	return mgr->priv->firsturi;
-}
-
-static void
-rb_playlist_manager_source_deleted_cb (RBSource *source, RBPlaylistManager *mgr)
-{
-	rb_debug ("removing playlist %p", source);
-	mgr->priv->playlists = g_list_remove (mgr->priv->playlists, source);
+	return TRUE;
 }
 
 static void
 append_new_playlist_source (RBPlaylistManager *mgr, RBPlaylistSource *source)
 {
-	mgr->priv->playlists = g_list_append (mgr->priv->playlists, source);
-	g_signal_connect_object (G_OBJECT (source), "deleted",
-				 G_CALLBACK (rb_playlist_manager_source_deleted_cb), mgr, 0);
 	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_ADDED], 0,
 		       source);
 }
@@ -524,7 +653,7 @@ rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 		if (xmlNodeIsText (child))
 			continue;
 
-		playlist = rb_playlist_source_new_from_xml (mgr->priv->db,
+		playlist = rb_playlist_source_new_from_xml (mgr->priv->shell,
 							    child);
 		append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
 	}
@@ -551,8 +680,12 @@ rb_playlist_manager_save_thread_main (struct RBPlaylistManagerSaveThreadData *da
 	file = g_build_filename (rb_dot_dir (), "playlists.xml", NULL);
 	tmpname = g_strconcat (file, ".tmp", NULL);
 	
-	xmlSaveFormatFile (tmpname, data->doc, 1);
-	rename (tmpname, file);
+	if (xmlSaveFormatFile (tmpname, data->doc, 1) != -1) {
+		rename (tmpname, file);
+	} else {
+		rb_debug ("error in xmlSaveFormatFile(), not saving");
+		unlink (tmpname);
+	}
 	xmlFreeDoc (data->doc);
 	g_free (tmpname);
 	g_free (file);
@@ -560,6 +693,7 @@ rb_playlist_manager_save_thread_main (struct RBPlaylistManagerSaveThreadData *da
 	g_mutex_lock (data->mgr->priv->saving_mutex);
 
 	data->mgr->priv->saving = FALSE;
+	data->mgr->priv->dirty = FALSE;
 
 	g_cond_signal (data->mgr->priv->saving_condition);
 	g_mutex_unlock (data->mgr->priv->saving_mutex);
@@ -574,17 +708,57 @@ rb_playlist_manager_save_thread_main (struct RBPlaylistManagerSaveThreadData *da
 void
 rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 {
-	GList *tmp;
 	xmlNodePtr root;
 	struct RBPlaylistManagerSaveThreadData *data;
+	GtkTreeIter iter;
+	GtkTreeModel *fmodel;
+	GtkTreeModel *model;
 
 	rb_debug ("saving the playlists");
 
-	if (!force && !mgr->priv->dirty) {
-		rb_debug ("no save needed, ignoring");
-		return;
-	}
+	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
+	g_object_unref (fmodel);
+	
+	if (!force) {
+		gboolean dirty = FALSE;
 
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			do {
+				RBSource *source;
+				gboolean local;
+				GValue v = {0,};
+				gtk_tree_model_get_value (model,
+							  &iter,
+							  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+							  &v);
+				source = g_value_get_pointer (&v);
+				if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
+					continue;
+
+				g_object_get (G_OBJECT (source), 
+					      "dirty", &dirty, 
+					      "is-local", &local, 
+					      NULL);
+				if (local && dirty) {
+					break;
+				}
+			} while (gtk_tree_model_iter_next (model, &iter));
+		}
+
+		if (!dirty) {
+			/* hmm, don't like taking saving_mutex twice like this */
+			g_mutex_lock (mgr->priv->saving_mutex);
+			dirty = mgr->priv->dirty;
+			g_mutex_unlock (mgr->priv->saving_mutex);
+		}
+
+		if (!dirty) {
+			rb_debug ("no save needed, ignoring");
+			return;
+		}
+	}
+	
 	g_mutex_lock (mgr->priv->saving_mutex);
 
 	while (mgr->priv->saving)
@@ -596,13 +770,33 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 
 	data = g_new0 (struct RBPlaylistManagerSaveThreadData, 1);
 	data->mgr = mgr;
-	data->doc = xmlNewDoc ("1.0");
+	data->doc = xmlNewDoc (RB_PLAYLIST_MGR_VERSION);
 
-	root = xmlNewDocNode (data->doc, NULL, "rhythmdb-playlists", NULL);
+	root = xmlNewDocNode (data->doc, NULL, RB_PLAYLIST_MGR_PL, NULL);
 	xmlDocSetRootElement (data->doc, root);
 	
-	for (tmp = mgr->priv->playlists; tmp; tmp = tmp->next)
-		rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (tmp->data), root);
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			RBSource *source;
+			GValue v = {0,};
+			gboolean local;
+
+			gtk_tree_model_get_value (model,
+						  &iter,
+						  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+						  &v);
+			source = g_value_get_pointer (&v);
+			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
+				continue;
+
+			g_object_get (G_OBJECT (source), "is-local", &local, 
+				      NULL);
+			if (local) {
+
+				rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (source), root);
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
 
 	g_object_ref (G_OBJECT (mgr));
 	mgr->priv->outstanding_threads++;
@@ -620,33 +814,64 @@ rb_playlist_manager_set_dirty (RBPlaylistManager *mgr)
 
 RBSource *
 rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
-				  const char *suggested_name, gboolean automatic)
+				  const char *suggested_name, 
+				  gboolean automatic)
 {
-	RBSource *playlist = RB_SOURCE (rb_playlist_source_new (mgr->priv->db, automatic));
-	GTimeVal serial;
-	char *internal;
-
-
-	g_get_current_time (&serial);
-	internal = g_strdup_printf ("<playlist:%ld:%ld>", serial.tv_sec,
-				    serial.tv_usec);
+	RBSource *playlist = RB_SOURCE (rb_playlist_source_new (mgr->priv->shell, automatic, TRUE));
 	
 	g_object_set (G_OBJECT (playlist),
 		      "name", suggested_name ? suggested_name : "",
-		      "internal-name", internal,
 		      NULL);
-	g_free (internal);
 	append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
 	rb_sourcelist_edit_source_name (mgr->priv->sourcelist, playlist);
 	rb_playlist_manager_set_dirty (mgr);
+	
+	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_CREATED], 0,
+		       playlist);
 
 	return playlist;
 }
 
+GList *
+rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
+{
+	GList *playlists = NULL;
+	GtkTreeIter iter;
+	GtkTreeModel *fmodel;
+	GtkTreeModel *model;
+	
+	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
+	g_object_unref (fmodel);
+	
+	if (gtk_tree_model_get_iter_first (model, &iter)) {
+		do {
+			RBSource *source;
+			GValue v = {0,};
+			gboolean local;
+
+			gtk_tree_model_get_value (model,
+						  &iter,
+						  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+						  &v);
+			source = g_value_get_pointer (&v);
+			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
+				continue;
+			g_object_get (G_OBJECT (source), "is-local", &local, 
+				      NULL);
+			if (local) {
+				playlists = g_list_prepend (playlists, source);
+			}
+
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+	
+	return playlists;
+}
+
 static void
-rb_playlist_manager_cmd_new_playlist (BonoboUIComponent *component,
-				      RBPlaylistManager *mgr,
-				      const char *verbname)
+rb_playlist_manager_cmd_new_playlist (GtkAction *action,
+				      RBPlaylistManager *mgr)
 {
 	rb_playlist_manager_new_playlist (mgr, NULL, FALSE);
 }
@@ -657,22 +882,31 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 					    RBQueryCreator *creator)
 {
 	RBQueryCreatorLimitType type;
-	guint limit, limit_count = 0, limit_size = 0;
+	guint limit, limit_count = 0, limit_size = 0, limit_time = 0;
+	const char *sort_key;
+	gint sort_direction;
 
 	rb_query_creator_get_limit (creator, &type, &limit);
 	if (type == RB_QUERY_CREATOR_LIMIT_COUNT)
 		limit_count = limit;
 	else if (type == RB_QUERY_CREATOR_LIMIT_MB)
 		limit_size = limit;
+	else if (type == RB_QUERY_CREATOR_LIMIT_SECONDS)
+		limit_time = limit;
+	else
+		g_assert_not_reached ();
+
+	rb_query_creator_get_sort_order (creator, &sort_key, &sort_direction);
+
 	rb_playlist_source_set_query (playlist,
 				      rb_query_creator_get_query (creator),
-				      limit_count, limit_size);
+				      limit_count, limit_size, limit_time,
+				      sort_key, sort_direction);
 }
 
 static void
-rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *component,
-						RBPlaylistManager *mgr,
-						const char *verbname)
+rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
+						RBPlaylistManager *mgr)
 {
 	RBQueryCreator *creator = RB_QUERY_CREATOR (rb_query_creator_new (mgr->priv->db));
 	RBSource *playlist;
@@ -695,23 +929,24 @@ rb_playlist_manager_cmd_new_automatic_playlist (BonoboUIComponent *component,
 }
 
 static void
-rb_playlist_manager_cmd_edit_automatic_playlist (BonoboUIComponent *component,
-						 RBPlaylistManager *mgr,
-						 const char *verbname)
+rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
+						 RBPlaylistManager *mgr)
 {
 	RBQueryCreator *creator;
 	RBPlaylistSource *playlist;
 	GPtrArray *query;
-	guint limit_count = 0, limit_size = 0;
+	guint limit_count = 0, limit_size = 0, limit_time = 0;
+	const char *sort_key;
+	gint sort_direction;
 	
 
 	playlist = RB_PLAYLIST_SOURCE (mgr->priv->selected_source);
-	rb_playlist_source_get_query (playlist, &query, &limit_count, &limit_size);
+	rb_playlist_source_get_query (playlist, &query, &limit_count, &limit_size, &limit_time, &sort_key, &sort_direction);
 
 	creator = RB_QUERY_CREATOR (rb_query_creator_new_from_query (mgr->priv->db,
 								     query,
-								     limit_count,
-								     limit_size));	
+								     limit_count, limit_size, limit_time,
+								     sort_key, sort_direction));
 
 	gtk_dialog_run (GTK_DIALOG (creator));
 
@@ -720,9 +955,18 @@ rb_playlist_manager_cmd_edit_automatic_playlist (BonoboUIComponent *component,
 }
 
 static void
-rb_playlist_manager_cmd_delete_playlist (BonoboUIComponent *component,
-					 RBPlaylistManager *mgr,
-					 const char *verbname)
+rb_playlist_manager_cmd_rename_playlist (GtkAction *action,
+					 RBPlaylistManager *mgr)
+{
+	rb_debug ("Renaming playlist %p", mgr->priv->selected_source);
+
+	rb_sourcelist_edit_source_name (mgr->priv->sourcelist, mgr->priv->selected_source);
+	rb_playlist_manager_set_dirty (mgr);
+}
+
+static void
+rb_playlist_manager_cmd_delete_playlist (GtkAction *action,
+					 RBPlaylistManager *mgr)
 {
 	rb_debug ("Deleting playlist %p", mgr->priv->selected_source);
 	
@@ -736,42 +980,40 @@ load_playlist_response_cb (GtkDialog *dialog,
 			   RBPlaylistManager *mgr)
 {
 	char *escaped_file = NULL;
+	GError *error = NULL;
 
-	if (response_id != GTK_RESPONSE_OK) {
+	if (response_id != GTK_RESPONSE_ACCEPT) {
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		return;
 	}
 
-#ifndef HAVE_GTK_2_3
-	{
-	char *file = NULL;
-	file = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)));
-	escaped_file = gnome_vfs_get_uri_from_local_path (file);
-	g_free (file);
-	}
-#else
 	escaped_file = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-#endif
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	if (escaped_file == NULL)
 		return;
 
-	rb_playlist_manager_parse_file (mgr, escaped_file);
+	if (!rb_playlist_manager_parse_file (mgr, escaped_file, &error)) {
+		rb_error_dialog (NULL, _("Couldn't read playlist"),
+				 error->message);
+		g_error_free (error);
+	}
+
 	g_free (escaped_file);
 	rb_playlist_manager_set_dirty (mgr);
 }
 
 static void
-rb_playlist_manager_cmd_load_playlist (BonoboUIComponent *component,
-				       RBPlaylistManager *mgr,
-				       const char *verbname)
+rb_playlist_manager_cmd_load_playlist (GtkAction *action,
+				       RBPlaylistManager *mgr)
 {
 	GtkWidget *dialog;
 
-	dialog = rb_ask_file (_("Load playlist"), NULL,
-			      GTK_WINDOW (mgr->priv->window));
+	dialog = rb_file_chooser_new (_("Load Playlist"),
+				      GTK_WINDOW (mgr->priv->window),
+				      GTK_FILE_CHOOSER_ACTION_OPEN,
+				      TRUE);
 
 	g_signal_connect_object (G_OBJECT (dialog), "response",
 				 G_CALLBACK (load_playlist_response_cb), mgr, 0);
@@ -782,18 +1024,14 @@ save_playlist_response_cb (GtkDialog *dialog,
 			   int response_id,
 			   RBPlaylistManager *mgr)
 {
-	char *file;
+	char *file = NULL;
 
-	if (response_id != GTK_RESPONSE_OK) {
+	if (response_id != GTK_RESPONSE_ACCEPT) {
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 		return;
 	}
 
-#ifndef HAVE_GTK_2_3
-	file = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)));
-#else
 	file = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-#endif
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
@@ -805,14 +1043,15 @@ save_playlist_response_cb (GtkDialog *dialog,
 }
 
 static void
-rb_playlist_manager_cmd_save_playlist (BonoboUIComponent *component,
-				       RBPlaylistManager *mgr,
-				       const char *verbname)
+rb_playlist_manager_cmd_save_playlist (GtkAction *action,
+				       RBPlaylistManager *mgr)
 {
 	GtkWidget *dialog;
     
-	dialog = rb_ask_file_save (_("Save playlist"), NULL,
-				   GTK_WINDOW (mgr->priv->window));
+	dialog = rb_file_chooser_new (_("Save Playlist"),
+				      GTK_WINDOW (mgr->priv->window),
+				      GTK_FILE_CHOOSER_ACTION_SAVE,
+				      TRUE);
 
 	g_signal_connect_object (G_OBJECT (dialog), "response",
 				 G_CALLBACK (save_playlist_response_cb),
@@ -820,64 +1059,17 @@ rb_playlist_manager_cmd_save_playlist (BonoboUIComponent *component,
 }
 
 static void
-add_uri_to_playlist (RBPlaylistManager *mgr, RBPlaylistSource *playlist, const char *uri, const char *title)
+rb_playlist_manager_cmd_burn_playlist (GtkAction *action,
+				       RBPlaylistManager *mgr)
 {
-	GError *error = NULL;
-	GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
-	const char *scheme = gnome_vfs_uri_get_scheme (vfsuri);
-
-	if (rb_uri_is_iradio (scheme)) {
-		rb_iradio_source_add_station (mgr->priv->iradio_source, uri, title, NULL);
-		goto out;
-	}
-
-	rhythmdb_add_song (mgr->priv->db, uri, &error);
-	if (error) {
-		rb_debug ("error loading URI %s", uri);
-		goto out; /* FIXME */
-	}
-
-	rb_playlist_source_add_location (playlist, uri);
-out:
-	gnome_vfs_uri_unref (vfsuri);
+	rb_playlist_source_burn_playlist (RB_PLAYLIST_SOURCE (mgr->priv->selected_source));
 }
 
 static void
-handle_playlist_entry_into_playlist_cb (RBPlaylist *playlist, const char *uri_maybe,
-					const char *title,
-					const char *genre, RBPlaylistManager *mgr)
+rb_playlist_manager_playlist_entries_changed (RBEntryView *entry_view, RhythmDBEntry *entry, RBPlaylistManager *mgr)
 {
-	char *uri;
+	int num_tracks = rb_entry_view_get_num_entries (entry_view);
+	GtkAction *action = gtk_action_group_get_action (mgr->priv->actiongroup, "MusicPlaylistBurnPlaylist");
 
-	if (uri_maybe[0] == '/') {
-		uri = gnome_vfs_get_uri_from_local_path (uri_maybe);
-		if (uri == NULL) {
-			rb_debug ("Error processing absolute filename: %s", uri_maybe);
-			return;
-		}
-	} else {
-		GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri_maybe);
-		if (!vfsuri) {
-			rb_debug ("Error processing probable URI: %s", uri_maybe);
-			return;
-		}
-		uri = gnome_vfs_uri_to_string (vfsuri, GNOME_VFS_URI_HIDE_NONE);
-		gnome_vfs_uri_unref (vfsuri);
-	}
-	
-	if (!mgr->priv->firsturi)
-		mgr->priv->firsturi = g_strdup (uri);
-	if (rb_uri_is_iradio (uri)) {
-		rb_iradio_source_add_station (mgr->priv->iradio_source,
-					      uri, title, genre);
-		g_free (uri);
-		return;
-	}
-
-	if (!mgr->priv->loading_playlist) {
-		mgr->priv->loading_playlist =
-			RB_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
-	}
-	add_uri_to_playlist (mgr, mgr->priv->loading_playlist, uri, title);
-	g_free (uri);
+	g_object_set (G_OBJECT (action), "sensitive", (num_tracks > 0), NULL);
 }
