@@ -51,7 +51,7 @@
 #include "rb-header.h"
 #include "totem-pl-parser.h"
 #include "rb-metadata.h"
-#include "rb-volume.h"
+#include "bacon-volume.h"
 #include "rb-remote.h"
 #include "rb-iradio-source.h"
 #include "rb-library-source.h"
@@ -89,8 +89,6 @@ static void rb_shell_player_get_property (GObject *object,
 					  GValue *value,
 					  GParamSpec *pspec);
 
-static void rb_shell_player_do_next_with_rating (RBShellPlayer *player,
-						 gboolean       do_rating);
 static void rb_shell_player_cmd_previous (GtkAction *action,
 			                  RBShellPlayer *player);
 static void rb_shell_player_cmd_play (GtkAction *action,
@@ -99,13 +97,18 @@ static void rb_shell_player_cmd_pause (GtkAction *action,
 			               RBShellPlayer *player);
 static void rb_shell_player_cmd_stop (GtkAction *action,
 			              RBShellPlayer *player);
-static void rb_shell_player_playpause_button (RBShellPlayer *player);
+static void rb_shell_player_playpause_button_cb (GtkButton *button,
+						  RBShellPlayer *player);
 static void rb_shell_player_cmd_next (GtkAction *action,
 			              RBShellPlayer *player);
 static void rb_shell_player_shuffle_changed_cb (GtkAction *action,
 						RBShellPlayer *player);
 static void rb_shell_player_repeat_changed_cb (GtkAction *action,
 					       RBShellPlayer *player);
+static void rb_shell_player_previous_button_cb (GtkButton *button,
+						RBShellPlayer *player);
+static void rb_shell_player_next_button_cb (GtkButton *button,
+					    RBShellPlayer *player);
 static void rb_shell_player_cmd_song_info (GtkAction *action,
 					   RBShellPlayer *player);
 static void rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
@@ -115,7 +118,11 @@ static void rb_shell_player_set_play_button (RBShellPlayer *player,
 			                     PlayButtonState state);
 static void rb_shell_player_sync_with_source (RBShellPlayer *player);
 static void rb_shell_player_sync_with_selected_source (RBShellPlayer *player);
-
+static void rb_shell_player_entry_changed_cb (RhythmDB *db,
+							RhythmDBEntry *entry,
+				       		GSList *changes,
+				       		RBShellPlayer *player);
+				       
 static void rb_shell_player_playing_entry_deleted_cb (RBEntryView *view,
 						      RhythmDBEntry *entry,
 						      RBShellPlayer *playa);
@@ -131,15 +138,16 @@ static void rb_shell_player_volume_changed_cb (GConfClient *client,
 					       guint cnxn_id,
 					       GConfEntry *entry,
 					       RBShellPlayer *playa);
+static void rb_shell_player_volume_widget_changed_cb (BaconVolumeButton *vol,
+						      RBShellPlayer     *playa);
 static void rb_shell_player_sync_volume (RBShellPlayer *player); 
 static void rb_shell_player_sync_replaygain (RBShellPlayer *player,
                                              RhythmDBEntry *entry);
 static void tick_cb (RBPlayer *player, long elapsed, gpointer data);
 static void eos_cb (RBPlayer *player, gpointer data);
-static void error_cb (RBPlayer *player, GError *err, gpointer data);
+static void error_cb (RBPlayer *player, const GError *err, gpointer data);
 static void buffering_cb (RBPlayer *player, guint progress, gpointer data);
-static void rb_shell_player_error (RBShellPlayer *player, GError *err,
-				   gboolean lock);
+static void rb_shell_player_error (RBShellPlayer *player, gboolean async, const GError *err);
 
 static void info_available_cb (RBPlayer *player,
                                RBMetaDataField field,
@@ -164,16 +172,6 @@ static void rb_shell_player_init_mmkeys (RBShellPlayer *shell_player);
 
 #define CONF_STATE		CONF_PREFIX "/state"
 
-#define AUTO_RATE_FIRST_SKIP_RATE        -0.40
-#define AUTO_RATE_CONS_SKIP_RATE         -0.27
-#define AUTO_RATE_FIRST_NON_SKIP_RATE     0.33
-#define AUTO_RATE_CONS_NON_SKIP_RATE      0.07
-#define AUTO_RATE_NO_XIDLE_BONUS          0.07
-#define AUTO_RATE_INTERACTIVE_BONUS       0.13
-#define AUTO_RATE_JUMPED_FROM            -0.07
-#define AUTO_RATE_JUMPED_TO_SKIPPED       0.07
-#define AUTO_RATE_JUMPED_TO_FINISHED      0.47
-
 struct RBShellPlayerPrivate
 {
 	RhythmDB *db;
@@ -182,6 +180,9 @@ struct RBShellPlayerPrivate
 	
 	RBSource *selected_source;
 	RBSource *source;
+	RhythmDBEntry *playing_attempt_entry;
+	gboolean did_retry;
+	GTimeVal last_retry;
 
 	GtkUIManager *ui_manager;
 	GtkActionGroup *actiongroup;
@@ -197,7 +198,6 @@ struct RBShellPlayerPrivate
 
 	char *song;
 	gboolean have_url;
-	gboolean have_artist_album;
 	char *url;
 
 	gboolean have_previous_entry;
@@ -252,7 +252,9 @@ enum
 	WINDOW_TITLE_CHANGED,
 	DURATION_CHANGED,
 	PLAYING_SOURCE_CHANGED,
+	PLAYING_CHANGED,
 	PLAYING_SONG_CHANGED,
+	PLAYING_URI_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -293,10 +295,10 @@ static GtkActionEntry rb_shell_player_stop_action [] =
 
 static GtkToggleActionEntry rb_shell_player_toggle_entries [] =
 {
-	{ "ControlShuffle", RB_STOCK_SHUFFLE, N_("Sh_uffle"), "<control>U",
+	{ "ControlShuffle", NULL, N_("Sh_uffle"), "<control>U",
 	  N_("Play songs in a random order"),
 	  G_CALLBACK (rb_shell_player_shuffle_changed_cb) },
-	{ "ControlRepeat", RB_STOCK_REPEAT, N_("_Repeat"), "<control>R",
+	{ "ControlRepeat", NULL, N_("_Repeat"), "<control>R",
 	  N_("Play first song again after all songs are played"),
 	  G_CALLBACK (rb_shell_player_repeat_changed_cb) }
 };
@@ -445,6 +447,17 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 			      1,
 			      RB_TYPE_SOURCE);
 
+	rb_shell_player_signals[PLAYING_CHANGED] =
+		g_signal_new ("playing-changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBShellPlayerClass, playing_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__BOOLEAN,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_BOOLEAN);
+
 	rb_shell_player_signals[PLAYING_SONG_CHANGED] =
 		g_signal_new ("playing-song-changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -455,6 +468,17 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_POINTER);
+
+	rb_shell_player_signals[PLAYING_URI_CHANGED] =
+		g_signal_new ("playing-uri-changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBShellPlayerClass, playing_uri_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_STRING);
 }
 
 static GObject *
@@ -542,6 +566,13 @@ volume_pre_unmount_cb (GnomeVFSVolumeMonitor *monitor,
 	g_free (volume_mount_point);
 }
 
+static void
+reemit_playing_signal (RBShellPlayer *player, GParamSpec *pspec, gpointer data)
+{
+	g_signal_emit (player, rb_shell_player_signals[PLAYING_CHANGED], 0,
+		       rb_player_playing (player->priv->mmplayer));
+}
+
 static void
 rb_shell_player_init (RBShellPlayer *player)
 {
@@ -603,8 +634,6 @@ rb_shell_player_init (RBShellPlayer *player)
 					    (GConfClientNotifyFunc)gconf_play_order_changed,
 					    player);
 
-	rb_shell_player_sync_volume (player);
-
 	hbox = gtk_hbox_new (FALSE, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
 
@@ -617,8 +646,8 @@ rb_shell_player_init (RBShellPlayer *player)
 
 	player->priv->prev_button = gtk_button_new ();
 	gtk_container_add (GTK_CONTAINER (player->priv->prev_button), image);
-	g_signal_connect_swapped (G_OBJECT (player->priv->prev_button),
-				  "clicked", G_CALLBACK (rb_shell_player_do_previous), player);
+	g_signal_connect (G_OBJECT (player->priv->prev_button),
+			  "clicked", G_CALLBACK (rb_shell_player_previous_button_cb), player);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
 			      GTK_WIDGET (player->priv->prev_button), 
 			      _("Play previous song"), NULL);
@@ -638,16 +667,16 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_container_add (GTK_CONTAINER (player->priv->play_pause_stop_button), player->priv->play_image);
 	player->priv->playbutton_state = PLAY_BUTTON_PLAY;
 
-	g_signal_connect_swapped (G_OBJECT (player->priv->play_pause_stop_button),
-				  "clicked", G_CALLBACK (rb_shell_player_playpause_button), player);
+	g_signal_connect (G_OBJECT (player->priv->play_pause_stop_button),
+			  "clicked", G_CALLBACK (rb_shell_player_playpause_button_cb), player);
 
 	/* Next button */
 	image = gtk_image_new_from_stock (GTK_STOCK_MEDIA_NEXT,
 					 GTK_ICON_SIZE_LARGE_TOOLBAR);
 	player->priv->next_button = gtk_button_new ();
 	gtk_container_add (GTK_CONTAINER (player->priv->next_button), image);
-	g_signal_connect_swapped (G_OBJECT (player->priv->next_button),
-				  "clicked", G_CALLBACK (rb_shell_player_do_next), player);
+	g_signal_connect (G_OBJECT (player->priv->next_button),
+			  "clicked", G_CALLBACK (rb_shell_player_next_button_cb), player);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
 			      GTK_WIDGET (player->priv->next_button), 
 			      _("Play next song"), NULL);
@@ -665,7 +694,12 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_container_add (GTK_CONTAINER (alignment), GTK_WIDGET (player->priv->header_widget));
 	gtk_box_pack_start (GTK_BOX (player), alignment, TRUE, TRUE, 0);
 
-	player->priv->volume_button = GTK_WIDGET (rb_volume_new ());
+	player->priv->volume_button = bacon_volume_button_new (GTK_ICON_SIZE_LARGE_TOOLBAR,
+							       0.0, 1.0, 0.02);
+	rb_shell_player_sync_volume (player);
+	g_signal_connect (player->priv->volume_button, "value-changed",
+			  G_CALLBACK (rb_shell_player_volume_widget_changed_cb),
+			  player);
 
 	gtk_tooltips_set_tip (GTK_TOOLTIPS (player->priv->tooltips), 
 			      GTK_WIDGET (player->priv->volume_button), 
@@ -680,6 +714,9 @@ rb_shell_player_init (RBShellPlayer *player)
 					    (GConfClientNotifyFunc) rb_shell_player_volume_changed_cb,
 					    player);
 
+	g_signal_connect (player, "notify::playing",
+			  G_CALLBACK (reemit_playing_signal), NULL);
+				 
 #ifdef HAVE_MMKEYS
 	/* Enable Multimedia Keys */
 	rb_shell_player_init_mmkeys (player);
@@ -698,6 +735,9 @@ rb_shell_player_finalize (GObject *object)
 
 	g_return_if_fail (player->priv != NULL);
 
+	if (player->priv->playing_attempt_entry)
+		rhythmdb_entry_unref (player->priv->db, player->priv->playing_attempt_entry);
+
 	eel_gconf_notification_remove(player->priv->gconf_play_order_id);
 	eel_gconf_notification_remove(player->priv->gconf_state_id);
 
@@ -785,6 +825,12 @@ rb_shell_player_set_property (GObject *object,
 		break;
 	case PROP_DB:
 		player->priv->db = g_value_get_object (value);
+		
+		/* Listen for changed entries to update metadata display */
+		g_signal_connect_object (G_OBJECT (player->priv->db),
+			 "entry_changed",
+			 G_CALLBACK (rb_shell_player_entry_changed_cb),
+			 player, 0);
 		break;
 	case PROP_ACTION_GROUP:
 		player->priv->actiongroup = g_value_get_object (value);
@@ -894,8 +940,9 @@ rb_shell_player_new (RhythmDB *db, GtkUIManager *mgr,
 RhythmDBEntry *
 rb_shell_player_get_playing_entry (RBShellPlayer *player)
 {
-	RBEntryView *songs;
+	
 	if (player->priv->source) {
+		RBEntryView *songs;
 		songs = rb_source_get_entry_view (player->priv->source);
 		return rb_entry_view_get_playing_entry (songs);
 	}
@@ -923,8 +970,7 @@ rb_shell_player_open_playlist_location (TotemPlParser *playlist, const char *uri
 	if (rb_player_playing (player->priv->mmplayer))
 		return;
 
-	rb_player_open (player->priv->mmplayer, uri, &error);
-	if (error != NULL) {
+	if (!rb_player_open (player->priv->mmplayer, uri, &error)) {
 		if (player->priv->playlist_parse_error != NULL) {
 			g_error_free (player->priv->playlist_parse_error);
 			player->priv->playlist_parse_error = NULL;
@@ -940,7 +986,7 @@ rb_shell_player_open_playlist_location (TotemPlParser *playlist, const char *uri
 	g_object_notify (G_OBJECT (player), "playing");
 }
 
-static void
+static gboolean
 rb_shell_player_open_location (RBShellPlayer *player,
 			       const char *location,
 			       GError **error)
@@ -957,9 +1003,8 @@ rb_shell_player_open_location (RBShellPlayer *player,
 
 	was_playing = rb_player_playing (player->priv->mmplayer);
 
-	rb_player_close (player->priv->mmplayer, error);
-	if (error && *error)
-		return;
+	if (!rb_player_close (player->priv->mmplayer, error))
+		return FALSE;
 
 	g_free (player->priv->song);
 	player->priv->song = NULL;
@@ -971,14 +1016,15 @@ rb_shell_player_open_location (RBShellPlayer *player,
 	totem_pl_parser_add_ignored_mimetype (playlist, "x-directory/normal");
 
 	playlist_parsed = FALSE;
-	if (rb_uri_is_iradio (location)) {
+	if (rb_source_try_playlist (player->priv->source)) {
 		playlist_result = totem_pl_parser_parse (playlist, location, FALSE);
 		playlist_parsed = (playlist_result != TOTEM_PL_PARSER_RESULT_UNHANDLED);
 	}
 	g_object_unref (playlist);
 	if (!playlist_parsed) {
 		/* We get here if we failed to parse as a playlist */
-		rb_player_open (player->priv->mmplayer, location, error);
+		if (!rb_player_open (player->priv->mmplayer, location, error))
+			return FALSE;
 	} else if (playlist_result == TOTEM_PL_PARSER_RESULT_ERROR
 		   && error
 		   && player->priv->playlist_parse_error) {
@@ -989,44 +1035,38 @@ rb_shell_player_open_location (RBShellPlayer *player,
 		 */
 		;
 	}
-	if (error && *error)
-		return;
 
 	rb_statusbar_set_progress (player->priv->statusbar_widget, -1.0, NULL);
 	
-	rb_player_play (player->priv->mmplayer, error);
-	if (error && *error)
-		return;
+	if (!rb_player_play (player->priv->mmplayer, error))
+		return FALSE;
 
 	if (!was_playing) {
 		g_object_notify (G_OBJECT (player), "playing");
 	}
+	return TRUE;
 }
 
-static void
+static gboolean
 rb_shell_player_open_entry (RBShellPlayer *player, RhythmDBEntry *entry, GError **error)
 {
-	rb_shell_player_open_location (player, entry->location, error);
-	if (error == NULL || *error == NULL)
-		return;
-	fprintf (stderr, "Got error opening \"%s\": %s\n",
-		 entry->location, (*error)->message);
+	return rb_shell_player_open_location (player, entry->location, error);
 }
 
-
-static void
+static gboolean
 rb_shell_player_play (RBShellPlayer *player, GError **error)
 {
 	RBEntryView *songs = rb_source_get_entry_view (player->priv->selected_source);
 
-	rb_player_play (player->priv->mmplayer, error);
-	if (error && *error)
-		return;
+	if (!rb_player_play (player->priv->mmplayer, error))
+		return FALSE;
 
 	rb_entry_view_set_playing (songs, TRUE);
 
 	rb_shell_player_sync_with_source (player);
 	rb_shell_player_sync_buttons (player);
+
+	return TRUE;
 }
 
 static void
@@ -1051,46 +1091,69 @@ rb_shell_player_set_entry_playback_error (RBShellPlayer *player,
 static gboolean
 do_next_idle (RBShellPlayer *player)
 {
-	rb_shell_player_do_next_with_rating (player, FALSE);
+	GError *error = NULL;
+	
+	if (!rb_shell_player_do_next (player, &error)) {
+		if (error->domain == RB_SHELL_PLAYER_ERROR
+		    && error->code == RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST) {
+			rb_debug ("No next entry, stopping playback");
+			rb_shell_player_set_playing_source (player, NULL);
+			g_object_notify (G_OBJECT (player), "playing");
+		}
+		g_warning ("do_next_idle: Unhandled error: %s", error->message);
+	}
 
 	return FALSE;
 }
 
-static void
-rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry)
+static gboolean
+rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry, GError **error)
 {
-	GError *error = NULL;
 	RBEntryView *songs;
+	GError *tmp_error = NULL;
 	
-	g_return_if_fail (player->priv->source != NULL);
-	g_return_if_fail (entry != NULL);
+	g_return_val_if_fail (player->priv->source != NULL, TRUE);
+	g_return_val_if_fail (entry != NULL, TRUE);
 	
 	songs = rb_source_get_entry_view (player->priv->source);
 
-	rb_shell_player_open_entry (player, entry, &error);
-	if (error == NULL) {
-		rb_shell_player_sync_replaygain (player, entry);
-		rb_shell_player_play (player, &error);
-	}
+	if (player->priv->playing_attempt_entry)
+		rhythmdb_entry_unref (player->priv->db, player->priv->playing_attempt_entry);
+	rhythmdb_entry_ref (player->priv->db, entry);
+	player->priv->playing_attempt_entry = entry;
+	if (!rb_shell_player_open_entry (player, entry, &tmp_error))
+		goto lose;
+	rb_shell_player_sync_replaygain (player, entry);
+	if (!rb_shell_player_play (player, &tmp_error))
+		goto lose;
 
 	rb_entry_view_set_playing_entry (songs, entry);
 
-	if (error != NULL) {
-		rb_shell_player_set_entry_playback_error (player, entry, error->message);
+	rb_debug ("Success!");
+	/* clear error on successful playback */
+	g_free (entry->playback_error);
+	entry->playback_error = NULL;
 
-		return;
-	} else {
-		rb_debug ("Success!");
-		/* clear error on successful playback */
-		g_free (entry->playback_error);
-		entry->playback_error = NULL;
-	}
 	g_signal_emit (G_OBJECT (player),
 		       rb_shell_player_signals[PLAYING_SONG_CHANGED], 0,
 		       entry);
+	g_signal_emit (G_OBJECT (player),
+		       rb_shell_player_signals[PLAYING_URI_CHANGED], 0,
+		       entry->location);
 
 	rb_shell_player_sync_with_source (player);
 	rb_shell_player_sync_buttons (player);
+
+	return TRUE;
+ lose:
+	/* Ignore errors, shutdown the player */
+	rb_player_close (player->priv->mmplayer, NULL);
+	/* Mark this song as failed */
+	rb_shell_player_set_entry_playback_error (player, entry, tmp_error->message);
+	g_propagate_error (error, tmp_error);
+	rhythmdb_entry_unref (player->priv->db, player->priv->playing_attempt_entry);
+	player->priv->playing_attempt_entry = NULL;
+	return FALSE;
 }
 
 static void
@@ -1192,180 +1255,100 @@ rb_shell_player_jump_to_current (RBShellPlayer *player)
 	rb_entry_view_select_entry (songs, rb_entry_view_get_playing_entry (songs));
 }
 
-void
-rb_shell_player_do_previous (RBShellPlayer *player)
+gboolean
+rb_shell_player_do_previous (RBShellPlayer *player, GError **error)
 {
 	RhythmDBEntry* entry;
 
 	if (player->priv->source != NULL) {
-		if (rb_source_can_pause (player->priv->source)) {
-			/* If we're in the first 2 seconds go to the previous song,
-			 * else restart the current one.
-			 */
-			if (rb_player_get_time (player->priv->mmplayer) < 3) {
-				rb_debug ("doing previous");
-				entry = rb_play_order_get_previous (player->priv->play_order);
-				if (entry) {
-					rb_play_order_go_previous (player->priv->play_order);
-					rb_shell_player_set_playing_entry (player, entry);
-				} else {
-					/* Is this the right thing to do when there's no previous song? */
-					rb_debug ("No previous entry, restarting song");
-					rb_player_set_time (player->priv->mmplayer, 0);
-					rb_header_sync_time (player->priv->header_widget);
-				}
-			} else {
-				rb_debug ("restarting song");
-				rb_player_set_time (player->priv->mmplayer, 0);
-				rb_header_sync_time (player->priv->header_widget);
-			}
-		} else {
-			rb_debug("source can't pause, going to previous");
-			entry = rb_play_order_get_previous (player->priv->play_order);
+		rb_debug ("going to previous");
+
+		entry = rb_play_order_get_previous (player->priv->play_order);
 			
-			if (entry) {
-				rb_debug ("previous station found, doing previous");
-				rb_play_order_go_previous (player->priv->play_order);
-				rb_shell_player_set_playing_entry (player, entry);
-			}
+		if (entry) {
+			rb_debug ("previous song found, doing previous");
+			rb_play_order_go_previous (player->priv->play_order);
+			if (!rb_shell_player_set_playing_entry (player, entry, error))
+				return FALSE;
+		} else {
+			rb_debug ("no previous song found, signaling error");
+			g_set_error (error,
+				     RB_SHELL_PLAYER_ERROR,
+				     RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST,
+				     _("No previous song"));
+			return FALSE;
 		}
-
+		
 		rb_shell_player_jump_to_current (player);
-	}
-}
-
-static void
-rb_shell_player_auto_adjust_rating (RBShellPlayer *player, gboolean jumped)
-{
-	RhythmDBEntry *current_entry;
-	long entry_duration, entry_play_time, entry_time_left;
-	gboolean check_idle_time;
-	double mod;
-	GValue value = { 0, };
-	
-	current_entry = rb_shell_player_get_playing_entry (player);
-	if (current_entry == NULL) {
-		player->priv->last_skipped = FALSE;
-		player->priv->last_jumped = jumped;
-		return;
-	}
-	
-	/* only auto-rate songs because our algorithm only works for discrete tunes */
-	if (current_entry->type != RHYTHMDB_ENTRY_TYPE_SONG)
-		return;
-	
-	entry_play_time = rb_shell_player_get_playing_time (player);
-	entry_duration = rb_shell_player_get_playing_song_duration (player);
-	entry_time_left = entry_duration - entry_play_time;
-	if (entry_time_left < 0)
-		entry_time_left = 0;
-	
-	if (entry_time_left == 0) {
-		if (player->priv->last_jumped)
-		        mod = AUTO_RATE_JUMPED_TO_FINISHED;
-		else if (player->priv->last_skipped)
-		        mod = AUTO_RATE_FIRST_NON_SKIP_RATE;
-		else
-		        mod = AUTO_RATE_CONS_NON_SKIP_RATE;
-		
-		check_idle_time = TRUE;
-		
-		player->priv->last_skipped = FALSE;
-		player->priv->last_jumped = FALSE;
+		return TRUE;
 	} else {
-		if (player->priv->last_jumped && !jumped)
-		        mod = AUTO_RATE_JUMPED_TO_SKIPPED;
-		else if (jumped)
-		        mod = AUTO_RATE_JUMPED_FROM;
-		else if (player->priv->last_skipped)
-		        mod = AUTO_RATE_CONS_SKIP_RATE;
-		else
-		        mod = AUTO_RATE_FIRST_SKIP_RATE;
-		
-		check_idle_time = mod < AUTO_RATE_JUMPED_FROM;
-		
-		player->priv->last_skipped = TRUE;
-		player->priv->last_jumped = jumped;
-	}
-	
-	if (check_idle_time) {
-#ifdef HAVE_XIDLE_EXTENSION
-		Display *display = gdk_x11_get_default_xdisplay ();
-		Time idle_time;
-		
-		/* get x11 idle time in milliseconds */
-		if (XGetIdleTime (display, &idle_time))
-			if ((idle_time / 1000) < entry_play_time)
-			        mod += AUTO_RATE_INTERACTIVE_BONUS;
-#else
-		mod += AUTO_RATE_NO_XIDLE_BONUS;
-#endif /* HAVE_XIDLE_EXTENSION */
-	}
-	
-	/* don't auto-rate songs 30 seconds or less */
-	if (eel_gconf_get_boolean (CONF_AUTO_RATE) && 
-	    current_entry->auto_rate &&
-	    entry_duration > 30) {
-		double old_rating, new_rating;
-		
-		/* get song's old rating */
-		old_rating = current_entry->rating;
-		
-		/* create and clamp new rating */
-		new_rating = old_rating + mod;
-		if (new_rating > 5)
-			new_rating = 5;
-		else if (new_rating < 0)
-			new_rating = 0;
-		
-		/* set the new value for the song */
-		g_value_init (&value, G_TYPE_DOUBLE);
-		g_value_set_double (&value, new_rating);
-		rhythmdb_entry_sync (player->priv->db,
-				    current_entry,
-				    RHYTHMDB_PROP_RATING,
-				    &value);
-		g_value_unset (&value);
-		rhythmdb_commit (player->priv->db);
-		
-		rb_debug ("set rating from %f to %f\n", old_rating, new_rating);
+		g_set_error (error,
+			     RB_SHELL_PLAYER_ERROR,
+			     RB_SHELL_PLAYER_ERROR_NOT_PLAYING,
+			     _("Not currently playing"));
+		return FALSE;
 	}
 }
 
-static void
-rb_shell_player_do_next_with_rating (RBShellPlayer *player,
-				     gboolean       do_rating)
+gboolean
+rb_shell_player_do_next (RBShellPlayer *player, GError **error)
 {
 	if (player->priv->source != NULL) {
 		RhythmDBEntry *entry = rb_play_order_get_next (player->priv->play_order);
-
-		if (do_rating)
-			rb_shell_player_auto_adjust_rating (player, FALSE);
 		
 		if (entry) {
 			rb_play_order_go_next (player->priv->play_order);
-			rb_shell_player_set_playing_entry (player, entry);
+			if (!rb_shell_player_set_playing_entry (player, entry, error))
+				return FALSE;
 		} else {
-			rb_debug ("No next entry, stopping playback");
-			rb_shell_player_set_playing_source (player, NULL);
-			g_object_notify (G_OBJECT (player), "playing");
+			g_set_error (error,
+				     RB_SHELL_PLAYER_ERROR,
+				     RB_SHELL_PLAYER_ERROR_END_OF_PLAYLIST,
+				     _("No next song"));
+			return FALSE;
 		}
 		rb_shell_player_jump_to_current (player);
 	}
+	return TRUE;
 }
 
-void
-rb_shell_player_do_next (RBShellPlayer *player)
+static void
+rb_shell_player_do_previous_or_seek (RBShellPlayer *player, GError **error)
 {
-	rb_shell_player_do_next_with_rating (player, TRUE);
+	rb_debug ("previous");
+	/* If we're in the first 3 seconds go to the previous song,
+	 * else restart the current one.
+	 */
+	if (player->priv->source != NULL
+	    && rb_source_can_pause (player->priv->source)
+	    && rb_player_get_time (player->priv->mmplayer) > 3) {
+		rb_debug ("after 3 second previous, restarting song");
+		rb_player_set_time (player->priv->mmplayer, 0);
+		rb_header_sync_time (player->priv->header_widget);
+	} else {
+		rb_shell_player_do_previous (player, NULL);
+	}
+}
+
+static void
+rb_shell_player_previous_button_cb (GtkButton *button,
+				    RBShellPlayer *player)
+{
+	rb_shell_player_do_previous_or_seek (player, NULL);
+}
+
+static void
+rb_shell_player_next_button_cb (GtkButton *button,
+				RBShellPlayer *player)
+{
+	rb_shell_player_do_next (player, NULL);
 }
 
 static void
 rb_shell_player_cmd_previous (GtkAction *action,
 			      RBShellPlayer *player)
 {
-	rb_debug ("previous");
-	rb_shell_player_do_previous (player);
+	rb_shell_player_do_previous_or_seek (player, NULL);
 }
 
 static void
@@ -1373,40 +1356,54 @@ rb_shell_player_cmd_next (GtkAction *action,
 			  RBShellPlayer *player)
 {
 	rb_debug ("next");
-	rb_shell_player_do_next (player);
+	rb_shell_player_do_next (player, NULL);
 }
 
 void
 rb_shell_player_play_entry (RBShellPlayer *player,
 			    RhythmDBEntry *entry)
 {
+	GError *error = NULL;
 	rb_shell_player_set_playing_source (player, player->priv->selected_source);
-	rb_shell_player_set_playing_entry (player, entry);
+	if (!rb_shell_player_set_playing_entry (player, entry, &error)) {
+		rb_shell_player_error (player, FALSE, error);
+		g_clear_error (&error);
+	}
 }
 
 static void
 rb_shell_player_cmd_play (GtkAction *action,
 			  RBShellPlayer *player)
 {
+	GError *error = NULL;
 	rb_debug ("play!");
-	rb_shell_player_playpause (player, TRUE);
+	if (!rb_shell_player_playpause (player, TRUE, &error))
+		rb_error_dialog (NULL, _("Couldn't start playback: %s"), error->message);
+	g_clear_error (&error);
 }
 
 static void
-rb_shell_player_playpause_button (RBShellPlayer *player)
+rb_shell_player_playpause_button_cb (GtkButton *button,
+				     RBShellPlayer *player)
 {
-	rb_shell_player_playpause (player, FALSE);
+	/* FIXME - do something with error... */
+	rb_shell_player_playpause (player, FALSE, NULL);
 }
 
-void
-rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop)
+gboolean
+rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop, GError **error)
 {
-	GError *error = NULL;
-	g_return_if_fail (RB_IS_SHELL_PLAYER (player));
+	gboolean ret;
+
+	rb_debug ("doing playpause");
+
+	g_return_val_if_fail (RB_IS_SHELL_PLAYER (player), TRUE);
+
+	ret = TRUE;
 
 	switch (player->priv->playbutton_state) {
 	case PLAY_BUTTON_STOP:
-		if (!ignore_stop) {
+		if (!ignore_stop || !rb_source_can_pause (player->priv->source)) {
 			rb_debug ("setting playing source to NULL");
 			rb_shell_player_set_playing_source (player, NULL);
 			break;
@@ -1439,12 +1436,13 @@ rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop)
 			}
 			if (entry != NULL) {
 				rb_play_order_go_next (player->priv->play_order);
-				rb_shell_player_set_playing_entry (player, entry);
+				if (!rb_shell_player_set_playing_entry (player, entry, error))
+					ret = FALSE;
 			}
 		} else {
-			rb_shell_player_play (player, &error);
-			if (error) {
+			if (!rb_shell_player_play (player, error)) {
 				rb_shell_player_set_playing_source (player, NULL);
+				ret = FALSE;
 			}
 		}
 	}
@@ -1455,14 +1453,16 @@ rb_shell_player_playpause (RBShellPlayer *player, gboolean ignore_stop)
 	rb_shell_player_sync_with_source (player);
 	rb_shell_player_sync_buttons (player);
 	g_object_notify (G_OBJECT (player), "playing");
+	return ret;
 }
 
 static void
 rb_shell_player_cmd_pause (GtkAction *action,
 			   RBShellPlayer *player)
 {
-	rb_debug ("This appears to be a mild setback for the stop faction");
-	rb_shell_player_playpause (player, TRUE);
+	rb_debug ("pausing");
+	/* FIXME handle error */
+	rb_shell_player_playpause (player, TRUE, NULL);
 
 }
 
@@ -1501,6 +1501,8 @@ rb_shell_player_sync_volume (RBShellPlayer *player)
 		volume = 0.0;
 	else if (volume > 1.0)
 		volume = 1.0;
+	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (player->priv->volume_button),
+				       volume);
 	rb_player_set_volume (player->priv->mmplayer,
 					volume);
 					
@@ -1550,6 +1552,14 @@ rb_shell_player_volume_changed_cb (GConfClient *client,
 	rb_shell_player_sync_volume (playa);
 }
 
+static void
+rb_shell_player_volume_widget_changed_cb (BaconVolumeButton *vol,
+					  RBShellPlayer     *playa)
+{
+	eel_gconf_set_float (CONF_STATE_VOLUME,
+			     bacon_volume_button_get_value (vol));
+}
+
 static void
 rb_shell_player_shuffle_changed_cb (GtkAction *action,
 				    RBShellPlayer *player)
@@ -1623,15 +1633,18 @@ rb_shell_player_entry_activated_cb (RBEntryView *view,
 				   RhythmDBEntry *entry,
 				   RBShellPlayer *playa)
 {
+	GError *error = NULL;
+
 	g_return_if_fail (entry != NULL);
 
 	rb_debug  ("got entry %p activated", entry);
 	
-	rb_shell_player_auto_adjust_rating (playa, TRUE);
-	
 	rb_shell_player_set_playing_source (playa, playa->priv->selected_source);
 
-	rb_shell_player_set_playing_entry (playa, entry);
+	if (!rb_shell_player_set_playing_entry (playa, entry, &error)) {
+		rb_shell_player_error (playa, FALSE, error);
+		g_clear_error (&error);
+	}
 }
 
 static void
@@ -1641,6 +1654,7 @@ rb_shell_player_property_row_activated_cb (RBPropertyView *view,
 {
 	RhythmDBEntry *entry;
 	RBEntryView *songs;
+	GError *error = NULL;
 
 	rb_debug  ("got property activated");
 	
@@ -1651,8 +1665,12 @@ rb_shell_player_property_row_activated_cb (RBPropertyView *view,
 	songs = rb_source_get_entry_view (playa->priv->source);
 	entry = rb_entry_view_get_first_entry (songs);
 
-	if (entry != NULL) {
-		rb_shell_player_set_playing_entry (playa, entry);
+	if (!entry)
+		return;
+
+	if (!rb_shell_player_set_playing_entry (playa, entry, &error)) {
+		rb_shell_player_error (playa, FALSE, error);
+		g_clear_error (&error);
 	}
 }
 
@@ -1697,6 +1715,35 @@ rb_shell_player_set_play_button (RBShellPlayer *player,
 	player->priv->playbutton_state = state;
 }
 
+static void
+rb_shell_player_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
+				       GSList *changes, RBShellPlayer *player)
+{
+	GSList *t;
+	RhythmDBEntry *playing_entry = rb_shell_player_get_playing_entry (player);
+	
+	/* We try to update only if the changed entry is currently playing */
+	if (entry != playing_entry) {
+		return;
+	}
+	
+	/* We update only if the artist, title or album has changed */
+	for (t = changes; t; t = t->next)
+	{
+		RhythmDBEntryChange *change = t->data;
+		switch (change->prop)
+		{
+			case RHYTHMDB_PROP_TITLE:
+			case RHYTHMDB_PROP_ARTIST:
+			case RHYTHMDB_PROP_ALBUM:
+				rb_shell_player_sync_with_source (player);
+				return;
+			default:
+				break;
+		}
+	}
+}
+
 static void
 rb_shell_player_sync_with_source (RBShellPlayer *player)
 {
@@ -1816,6 +1863,8 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 		else
 			pstate = PLAY_BUTTON_STOP;
 
+		rb_entry_view_set_playing (view, TRUE);
+
 		action = gtk_action_group_get_action (player->priv->play_action_group,
 						      "ControlPlay");
 		g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
@@ -1828,6 +1877,8 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 		else
 			pstate = PLAY_BUTTON_STOP;
 
+		rb_entry_view_set_playing (view, FALSE);
+
 		action = gtk_action_group_get_action (player->priv->play_action_group,
 						      "ControlPlay");
 		g_object_set (G_OBJECT (action),
@@ -1872,6 +1923,10 @@ rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
 	
 	player->priv->source = source;
 
+	if (player->priv->playing_attempt_entry)
+		rhythmdb_entry_unref (player->priv->db, player->priv->playing_attempt_entry);
+	player->priv->playing_attempt_entry = NULL;
+
 	if (source != NULL) {
 		RBEntryView *songs = rb_source_get_entry_view (player->priv->source);
 		g_signal_connect_object (G_OBJECT (songs),
@@ -1889,7 +1944,6 @@ rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
 	player->priv->song = NULL;
 	player->priv->url = NULL;
 	player->priv->have_url = FALSE;
-	player->priv->have_artist_album = FALSE;
 
 	if (source == NULL)
 		rb_shell_player_stop (player);
@@ -1991,9 +2045,6 @@ rb_shell_player_sync_with_selected_source (RBShellPlayer *player)
 
 		player->priv->have_url = rb_source_have_url (player->priv->selected_source);
 
-		player->priv->have_artist_album
-			= rb_source_have_artist_album (player->priv->selected_source);
-
 		rb_shell_player_sync_with_source (player);
 	}
 }
@@ -2020,8 +2071,23 @@ eos_cb (RBPlayer *mmplayer, gpointer data)
 					 _("Unexpected end of stream!"));
 			rb_shell_player_set_playing_source (player, NULL);
 			break;
+		case RB_SOURCE_EOF_RETRY: {
+			GTimeVal current;
+			g_get_current_time (&current);
+			if (player->priv->did_retry
+			    && (current.tv_sec - player->priv->last_retry.tv_sec) < 4) {
+				rb_debug ("Last retry was less than 4 seconds ago...aborting retry playback");
+				player->priv->did_retry = FALSE;
+				break;
+			} else {
+				player->priv->did_retry = TRUE;
+				g_get_current_time (&(player->priv->last_retry));
+				rb_shell_player_play_entry (player, entry);
+			}
+		}
+			break;
 		case RB_SOURCE_EOF_NEXT:
-			rb_shell_player_do_next (player);
+			rb_shell_player_do_next (player, NULL);
 			break;
 		}
 
@@ -2034,63 +2100,65 @@ eos_cb (RBPlayer *mmplayer, gpointer data)
 }
 
 static void
-rb_shell_player_error (RBShellPlayer *player, GError *err,
-		       gboolean lock)
+rb_shell_player_error (RBShellPlayer *player, gboolean async, const GError *err)
 {
 	RBEntryView *songs;
 	RhythmDBEntry *entry;
+	RhythmDBEntry *displayed_entry;
 
-	if (player->priv->source == NULL) {
-		rb_debug ("ignoring error (no source): %s", err->message);
-		return;
-	}
-
-	songs = rb_source_get_entry_view (player->priv->source);
-	entry = rb_entry_view_get_playing_entry (songs);
-
-	if (player->priv->handling_error) {
-		rb_debug ("ignoring error: %s", err->message);
-		return;
-	}
-
-	if (lock != FALSE)
-		GDK_THREADS_ENTER ();
+	g_assert (player->priv->handling_error == FALSE);
 
 	player->priv->handling_error = TRUE;
 
-	rb_debug ("error: %s", err->message);
-	rb_shell_player_set_entry_playback_error (player, entry, err->message);
+	entry = player->priv->playing_attempt_entry;
 
-	switch (rb_source_handle_eos (player->priv->source)) {
-	case RB_SOURCE_EOF_ERROR:
-		rb_error_dialog (NULL, _("Stream error"),
-				 _("Unexpected end of stream!"));
-		rb_shell_player_set_playing_source (player, NULL);
-		break;
-	case RB_SOURCE_EOF_NEXT:
-		if (err->code == RB_PLAYER_ERROR_NO_AUDIO)
-			rb_shell_player_set_playing_source (player, NULL);
-		else
-			g_idle_add ((GSourceFunc)do_next_idle, player);
-		break;
+	songs = rb_source_get_entry_view (player->priv->source);
+
+	displayed_entry = rb_entry_view_get_playing_entry (songs);
+	if (displayed_entry != entry) {
+		/* This can happen if there was an error - the currently
+		 * displayed entry won't match what we were trying to play.
+		 * In order to get the play order to actually go next, we
+		 * set the currently playing entry as a hack here.  This
+		 * should really be fixed so RBPlayOrder also tracks
+		 * the attempted entry, and just uses the view to decide
+		 * where the next/previous one is.
+		 */
+		rb_entry_view_set_playing_entry (songs, entry);
 	}
 
-	player->priv->handling_error = FALSE;
-	rb_debug ("exiting error hander");
+	g_printerr ("playback error: %s", err->message);
+	/* For synchronous errors the entry playback error has already been set */
+	if (entry && async)
+		rb_shell_player_set_entry_playback_error (player, entry, err->message);
 
-	if (lock != FALSE)
-		GDK_THREADS_LEAVE ();
+	if (err->code == RB_PLAYER_ERROR_NO_AUDIO)
+		rb_shell_player_set_playing_source (player, NULL);
+	else
+		g_idle_add ((GSourceFunc)do_next_idle, player);
+
+	player->priv->handling_error = FALSE;
 }
 
 static void
-error_cb (RBPlayer *mmplayer, GError *err, gpointer data)
+error_cb (RBPlayer *mmplayer, const GError *err, gpointer data)
 {
 	RBShellPlayer *player = RB_SHELL_PLAYER (data);
 
 	if (player->priv->handling_error)
 		return;
 
-	rb_shell_player_error (player, err, TRUE);
+	if (player->priv->source == NULL) {
+		rb_debug ("ignoring error (no source): %s", err->message);
+		return;
+	}
+
+	GDK_THREADS_ENTER ();
+
+	rb_shell_player_error (player, TRUE, err);
+	
+	rb_debug ("exiting error hander");
+	GDK_THREADS_LEAVE ();
 }
 
 static void
@@ -2234,7 +2302,7 @@ info_available_cb (RBPlayer *mmplayer,
 		rb_shell_player_sync_with_source (player);
 
 	if (set_field && entry_field != 0) {
-		rhythmdb_entry_sync (player->priv->db, entry, entry_field, value);
+		rhythmdb_entry_set (player->priv->db, entry, entry_field, value);
 		rhythmdb_commit (player->priv->db);
 	}
 
@@ -2320,21 +2388,21 @@ filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	player = (RBShellPlayer *)data;
 
 	if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay) == key->keycode) {	
-		rb_shell_player_playpause (player, TRUE);
+		rb_shell_player_playpause (player, TRUE, NULL);
 		return GDK_FILTER_REMOVE;
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {	
 		if (rb_shell_player_get_playing	(player)) {
-			rb_shell_player_playpause (player, TRUE);
+			rb_shell_player_playpause (player, TRUE, NULL);
 		}
 		return GDK_FILTER_REMOVE;
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop) == key->keycode) {
 		rb_shell_player_set_playing_source (player, NULL);
 		return GDK_FILTER_REMOVE;		
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev) == key->keycode) {
-		rb_shell_player_do_previous (player);
+		rb_shell_player_cmd_previous (NULL, player);
 		return GDK_FILTER_REMOVE;		
 	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext) == key->keycode) {
-		rb_shell_player_do_next (player);
+		rb_shell_player_cmd_next (NULL, player);
 		return GDK_FILTER_REMOVE;
 	} else {
 		return GDK_FILTER_CONTINUE;
