@@ -31,6 +31,9 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkradiobutton.h>
 #include <gtk/gtknotebook.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkcheckbutton.h>
 #include <glade/glade.h>
 #include <libgnome/gnome-help.h>
 #include <string.h>
@@ -83,6 +86,7 @@ struct RBShellPreferencesPrivate
 	GtkWidget *rating_check;
 	GtkWidget *play_count_check;
 	GtkWidget *last_played_check;
+	GtkWidget *first_seen_check;
 	GtkWidget *quality_check;
 
 	gboolean loading;
@@ -221,6 +225,8 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 		glade_xml_get_widget (xml, "last_played_check");
 	shell_preferences->priv->quality_check =
 		glade_xml_get_widget (xml, "quality_check");
+	shell_preferences->priv->first_seen_check =
+		glade_xml_get_widget (xml, "first_seen_check");
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
 				  glade_xml_get_widget (xml, "general_vbox"),
@@ -258,19 +264,90 @@ rb_shell_preferences_append_view_page (RBShellPreferences *prefs,
 				       RBSource *source)
 {
 	GtkWidget *widget;
+	GtkWidget *label;
+	gboolean visible;
 
 	g_return_if_fail (RB_IS_SHELL_PREFERENCES (prefs));
 	g_return_if_fail (RB_IS_SOURCE (source));
 
 	widget = rb_source_get_config_widget (source);
-	if (widget)
-		gtk_notebook_append_page (GTK_NOTEBOOK (prefs->priv->notebook),
-					  widget,
-					  gtk_label_new (name));
-	else
-		rb_debug ("No config widget for source %s", name);
+	if (!widget)
+		return;
+	label = gtk_label_new (name);
+	g_object_get (source, "visibility", &visible, NULL);
+	if (!visible)
+		gtk_widget_set_sensitive (label, FALSE);
+	gtk_notebook_append_page (GTK_NOTEBOOK (prefs->priv->notebook),
+				  widget,
+				  label);
 }
 
+#ifdef WITH_DAAP_SUPPORT
+
+#define CONF_ENABLE_SHARING "/apps/rhythmbox/sharing/enable_sharing"
+#define CONF_NAME "/apps/rhythmbox/sharing/share_name"
+
+static void
+share_check_button_toggled_cb (GtkToggleButton *button, GtkWidget *hbox)
+{
+	gboolean b;
+
+	b = gtk_toggle_button_get_active (button);
+
+	eel_gconf_set_boolean (CONF_ENABLE_SHARING, b);
+
+	gtk_widget_set_sensitive (hbox, b);
+	
+	return;
+}
+
+static gboolean
+share_name_entry_focus_out_event_cb (GtkEntry *entry, GdkEventFocus *event, gpointer data)
+{
+	const gchar *name;
+
+	name = gtk_entry_get_text (entry);
+
+	eel_gconf_set_string (CONF_NAME, name);
+
+	return FALSE;
+}
+
+static void
+add_daap_preferences (RBShellPreferences *shell_preferences)
+{
+	GladeXML *xml;
+	GtkWidget *check;
+	GtkWidget *entry;
+	GtkWidget *hbox;
+	gboolean b;
+
+	gchar *name;
+
+	xml = rb_glade_xml_new ("daap-prefs.glade",
+				"daap_vbox",
+				shell_preferences);
+
+	check = glade_xml_get_widget (xml, "daap_enable_check");
+	entry = glade_xml_get_widget (xml, "daap_name_entry");
+	hbox = glade_xml_get_widget (xml, "daap_hbox");
+	
+	b = eel_gconf_get_boolean (CONF_ENABLE_SHARING);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), b);
+	g_signal_connect (G_OBJECT (check), "toggled", G_CALLBACK (share_check_button_toggled_cb), hbox);
+	
+	name = eel_gconf_get_string (CONF_NAME);
+	gtk_entry_set_text (GTK_ENTRY (entry), name);
+	g_signal_connect (G_OBJECT (entry), "focus-out-event", G_CALLBACK (share_name_entry_focus_out_event_cb), NULL);	
+
+	gtk_widget_set_sensitive (hbox, b);
+	
+	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
+				  glade_xml_get_widget (xml, "daap_vbox"),
+				  gtk_label_new (_("Sharing")));
+}
+#endif /* WITH_DAAP_SUPPORT */
+
 GtkWidget *
 rb_shell_preferences_new (GList *views)
 {
@@ -290,6 +367,11 @@ rb_shell_preferences_new (GList *views)
 						       name,
 						       RB_SOURCE (views->data));
 	}
+
+#ifdef WITH_DAAP_SUPPORT
+	add_daap_preferences (shell_preferences);
+#endif /* WITH_DAAP_SUPPORT */
+
 	return GTK_WIDGET (shell_preferences);
 }
 
@@ -353,6 +435,8 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 	else if (shell_preferences->priv->quality_check
 		 && butt == GTK_CHECK_BUTTON (shell_preferences->priv->quality_check))
 		colname = "RHYTHMDB_PROP_BITRATE";
+	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->first_seen_check))
+		colname = "RHYTHMDB_PROP_FIRST_SEEN";
 	else
 		g_assert_not_reached ();
 
@@ -417,6 +501,9 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 			gtk_toggle_button_set_active
 				(GTK_TOGGLE_BUTTON (shell_preferences->priv->quality_check),
 				 strstr (columns, "RHYTHMDB_PROP_BITRATE") != NULL);
+		gtk_toggle_button_set_active
+			(GTK_TOGGLE_BUTTON (shell_preferences->priv->first_seen_check),
+			 strstr (columns, "RHYTHMDB_PROP_FIRST_SEEN") != NULL);
 	}
 
 	g_free (columns);
