@@ -227,24 +227,6 @@ struct playcount {
 
 static struct playcount *get_next_playcount (iPodParser *parser);
 
-#if 0
-#define DEFAULT_MOUNT_PATH "/mnt/ipod"
-#define GCONF_MOUNT_PATH "/apps/qahog/mount_path"
-
-
-static char *
-ipod_get_mount_path (char *mount_point)
-{
-	gchar *path;
-
-	path = eel_gconf_get_string (GCONF_MOUNT_PATH);
-	if (path == NULL || strcmp (path, "") == 0)
-		return g_strdup (DEFAULT_MOUNT_PATH);
-	else
-		return path;
-}
-#endif
-
 static char *
 ipod_get_itunesdb_path (const char *mount_path)
 {
@@ -305,7 +287,7 @@ static guint32 get4int(FILE *file, glong seek)
   guchar data[4];
   guint32 n;
 
-  if (seek_get_n_bytes (file, data, seek, 4) != 4) return -1;
+  if (seek_get_n_bytes (file, (gchar *)data, seek, 4) != 4) return -1;
   n =  ((guint32)data[3]) << 24;
   n += ((guint32)data[2]) << 16;
   n += ((guint32)data[1]) << 8;
@@ -399,8 +381,9 @@ static iPodPlaylist * get_pl(iPodParser *parser)
 #ifdef ITUNESDB_PROVIDE_UTF8
   gchar *plname_utf8;
 #endif
-  guint32 type, pltype, tracknum, n;
+  guint32 pltype, tracknum, n;
   guint32 nextseek;
+  gint32 type;
   gint32 zip;
   iPodPlaylist *plitem;
   guint32 ref;
@@ -491,7 +474,7 @@ static iPodPlaylist * get_pl(iPodParser *parser)
       if (cmp_n_bytes (data, "mhip", 4) == TRUE)
 	{
 	  ref = get4int(parser->itunes, parser->seek+24);
-	  plitem->song_ids = g_list_append (plitem->song_ids, (gpointer)ref);
+	  plitem->song_ids = g_list_append (plitem->song_ids, GUINT_TO_POINTER (ref));
 	  ++n;
 	}
       parser->seek += get4int (parser->itunes, parser->seek+8);
@@ -883,7 +866,7 @@ static gboolean
 ipod_parse_playlists (iPodParser *parser)
 {
     gboolean result = FALSE;
-    guchar data[8];
+    gchar data[8];
 
     do
     {
@@ -1058,6 +1041,11 @@ ipod_parser_new (const gchar *mount_point)
 	return parser;
 }
 
+gchar *
+ipod_parser_get_mount_path (iPodParser *parser)
+{
+	return g_strdup (parser->mount_path);
+}
 
 void
 ipod_parser_destroy (iPodParser *parser)
