@@ -40,7 +40,6 @@
 #include "rb-util.h"
 #include "rb-file-helpers.h"
 #include "rb-dialog.h"
-#include "rb-volume.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
@@ -112,7 +111,6 @@ static void rb_library_source_preferences_sync (RBLibrarySource *source);
 /* source methods */
 static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
-static const char *impl_get_search_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static GList *impl_get_extra_views (RBSource *source);
@@ -121,10 +119,9 @@ static void impl_search (RBSource *source, const char *text);
 static void impl_reset_filters (RBSource *source);
 static GtkWidget *impl_get_config_widget (RBSource *source);
 static void impl_song_properties (RBSource *source);
-static const char * impl_get_artist (RBSource *player);
-static const char * impl_get_album (RBSource *player);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_show_popup (RBSource *source);
+static const char * impl_get_paned_key (RBLibrarySource *source);
 static void rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype);
 
 void rb_library_source_browser_views_activated_cb (GtkWidget *widget,
@@ -144,7 +141,6 @@ static void songs_view_drag_data_received_cb (GtkWidget *widget,
 #define CONF_STATE_LIBRARY_SORTING CONF_PREFIX "/state/library/sorting"
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/library/paned_position"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/library/show_browser"
-#define CONF_STATE_SEARCH_TEXT   CONF_PREFIX "/state/library/search_text"
 
 struct RBLibrarySourcePrivate
 {
@@ -190,9 +186,12 @@ struct RBLibrarySourcePrivate
 	GSList *browser_views_group;
 	
 	RhythmDBEntryType entry_type;
-	
+
+	char *sorting_key;
 	guint ui_dir_notify_id;
-	guint state_dir_notify_id;
+	guint state_paned_notify_id;
+	guint state_browser_notify_id;
+	guint state_sorting_notify_id;
 	guint browser_view_notify_id;
 };
 
@@ -215,7 +214,8 @@ enum
 {
 	PROP_0,
 	PROP_ICON,
-	PROP_ENTRY_TYPE
+	PROP_ENTRY_TYPE,
+	PROP_SORTING_KEY
 };
 
 static GObjectClass *parent_class = NULL;
@@ -266,7 +266,6 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	source_class->impl_get_status = impl_get_status;
 	source_class->impl_get_browser_key = impl_get_browser_key;
-	source_class->impl_get_search_key = impl_get_search_key;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
@@ -280,13 +279,12 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_have_artist_album = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_get_artist = impl_get_artist;
-	source_class->impl_get_album = impl_get_album;
 	source_class->impl_have_url = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_show_popup = impl_show_popup;
 
+	klass->impl_get_paned_key = impl_get_paned_key;
+
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
 					 g_param_spec_object ("icon",
@@ -304,6 +302,14 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 							    RHYTHMDB_ENTRY_TYPE_SONG,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_SORTING_KEY,
+					 g_param_spec_string ("sorting-key",
+							      "Sorting key",
+							      "GConf key for storing sort-order",
+							      CONF_STATE_LIBRARY_SORTING,
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static void
@@ -438,9 +444,12 @@ rb_library_source_finalize (GObject *object)
 	rb_debug ("finalizing library source");
 	
 	eel_gconf_notification_remove (source->priv->ui_dir_notify_id);
-	eel_gconf_notification_remove (source->priv->state_dir_notify_id);
 	eel_gconf_notification_remove (source->priv->browser_view_notify_id);
+	eel_gconf_notification_remove (source->priv->state_browser_notify_id);
+	eel_gconf_notification_remove (source->priv->state_paned_notify_id);
+	eel_gconf_notification_remove (source->priv->state_sorting_notify_id);
 
+	g_free (source->priv->sorting_key);
 	g_free (source->priv->artist);
 	g_free (source->priv->album);
 
@@ -525,7 +534,7 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	source->priv->browser = gtk_hbox_new (TRUE, 5);
 
 	/* set up songs tree view */
-	source->priv->songs = rb_entry_view_new (source->priv->db, CONF_STATE_LIBRARY_SORTING,
+	source->priv->songs = rb_entry_view_new (source->priv->db, source->priv->sorting_key,
 						 TRUE, FALSE);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TRACK_NUMBER);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TITLE);
@@ -537,6 +546,7 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_LAST_PLAYED);
+	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_FIRST_SEEN);
 
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "show_popup",
 				 G_CALLBACK (rb_library_source_songs_show_popup_cb), source, 0);
@@ -617,16 +627,33 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	rb_library_source_state_prefs_sync (source);
 	rb_library_source_ui_prefs_sync (source);
 	update_browser_views_visibility (source);
+
+	if (source->priv->sorting_key) {
+		source->priv->state_sorting_notify_id =
+			eel_gconf_notification_add (source->priv->sorting_key,
+					    (GConfClientNotifyFunc) rb_library_source_state_pref_changed,
+					    source);
+	}
+	if (rb_library_source_get_paned_key (source)) {
+		source->priv->state_paned_notify_id =
+			eel_gconf_notification_add (rb_library_source_get_paned_key (source),
+					    (GConfClientNotifyFunc) rb_library_source_state_pref_changed,
+					    source);
+	}
+	if (rb_source_get_browser_key (RB_SOURCE (source))) {
+		source->priv->state_browser_notify_id =
+			eel_gconf_notification_add (rb_source_get_browser_key (RB_SOURCE (source)),
+					    (GConfClientNotifyFunc) rb_library_source_state_pref_changed,
+					    source);
+	}
+
 	source->priv->ui_dir_notify_id =
-		eel_gconf_notification_add (CONF_STATE_LIBRARY_DIR,
-				    (GConfClientNotifyFunc) rb_library_source_state_pref_changed,
-				    source);
-	source->priv->state_dir_notify_id =
 		eel_gconf_notification_add (CONF_UI_LIBRARY_DIR,
 				    (GConfClientNotifyFunc) rb_library_source_ui_pref_changed, source);
 	source->priv->browser_view_notify_id =
 		eel_gconf_notification_add (CONF_UI_LIBRARY_BROWSER_VIEWS,
 				    (GConfClientNotifyFunc) rb_library_source_browser_views_changed, source);
+
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
 	return G_OBJECT (source);
 }
@@ -647,11 +674,17 @@ rb_library_source_set_property (GObject *object,
 			g_object_unref (G_OBJECT (source->priv->pixbuf));
 		}
 		source->priv->pixbuf = g_value_get_object (value);
-		g_object_ref (source->priv->pixbuf);
+
+		if (source->priv->pixbuf)
+			g_object_ref (source->priv->pixbuf);
 		break;
 	case PROP_ENTRY_TYPE:
 		source->priv->entry_type = g_value_get_uint (value);
 		break;
+	case PROP_SORTING_KEY:
+		g_free (source->priv->sorting_key);
+		source->priv->sorting_key = g_strdup (g_value_get_string (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -674,6 +707,9 @@ rb_library_source_get_property (GObject *object,
 	case PROP_ENTRY_TYPE:
 		g_value_set_uint (value, source->priv->entry_type);
 		break;
+	case PROP_SORTING_KEY:
+		g_value_set_string (value, source->priv->sorting_key);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -684,17 +720,17 @@ RBSource *
 rb_library_source_new (RBShell *shell)
 {
 	RBSource *source;
-	GtkWidget *dummy = gtk_tree_view_new ();
 	GdkPixbuf *icon;
+	gint size;
 
-	icon = gtk_widget_render_icon (dummy, RB_STOCK_LIBRARY,
-				       GTK_ICON_SIZE_LARGE_TOOLBAR,
-				       NULL);
-	gtk_widget_destroy (dummy);
+	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+					 "stock_music-library",
+					 size,
+					 0, NULL);
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
 					  "name", _("Library"),
-					  "internal-name", "<library>",
 					  "entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
 					  "shell", shell,
 					  "icon", icon,
@@ -704,7 +740,6 @@ rb_library_source_new (RBShell *shell)
 	return source;
 }
 
-
 static gboolean
 string_list_equal (GList *a, GList *b)
 {
@@ -961,9 +996,9 @@ impl_get_browser_key (RBSource *source)
 }
 
 static const char *
-impl_get_search_key (RBSource *source)
+impl_get_paned_key (RBLibrarySource *status)
 {
-	return CONF_STATE_SEARCH_TEXT;
+	return CONF_STATE_PANED_POSITION;
 }
 
 static GdkPixbuf *
@@ -992,7 +1027,7 @@ impl_search (RBSource *asource, const char *search_text)
 	rb_debug ("doing search for \"%s\"", search_text ? search_text : "(NULL)");
 
 	g_free (source->priv->search_text);
-	source->priv->search_text = search_text != NULL ? g_utf8_casefold (search_text, -1) : NULL;
+	source->priv->search_text = g_strdup (search_text);
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_SEARCH);
 
 	rb_source_notify_filter_changed (RB_SOURCE (source));
@@ -1090,44 +1125,6 @@ impl_get_config_widget (RBSource *asource)
 	return source->priv->config_widget;
 }
 
-static const char *
-impl_get_artist (RBSource *asource)
-{
-	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	RhythmDBEntry *entry;
-
-	g_free (source->priv->artist);
-
-	entry = rb_entry_view_get_playing_entry (source->priv->songs);
-
-	if (entry != NULL) {
-		source->priv->artist = g_strdup (rb_refstring_get (entry->artist));
-	} else {
-		source->priv->artist = NULL;
-	}
-
-	return source->priv->artist;
-}
-
-static const char *
-impl_get_album (RBSource *asource)
-{
-	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	RhythmDBEntry *entry;
-
-	g_free (source->priv->artist);
-
-	entry = rb_entry_view_get_playing_entry (source->priv->songs);
-
-	if (entry != NULL) {
-		source->priv->album = g_strdup (rb_refstring_get (entry->album));
-	} else {
-		source->priv->album = NULL;
-	}
-	return source->priv->album;
-}
-
-
 static void
 rb_library_source_preferences_sync (RBLibrarySource *source)
 {
@@ -1189,23 +1186,30 @@ paned_size_allocate_cb (GtkWidget *widget,
 			GtkAllocation *allocation,
 		        RBLibrarySource *source)
 {
+	const char *key = rb_library_source_get_paned_key (source);;
+
 	/* save state */
 	rb_debug ("paned size allocate");
-	eel_gconf_set_integer (CONF_STATE_PANED_POSITION,
-			       gtk_paned_get_position (GTK_PANED (source->priv->paned)));
+	if (key)
+		eel_gconf_set_integer (key, gtk_paned_get_position (GTK_PANED (source->priv->paned)));
 }
 
 static void
 rb_library_source_state_prefs_sync (RBLibrarySource *source)
 {
+	const char *key;
+
 	rb_debug ("syncing state");
-	gtk_paned_set_position (GTK_PANED (source->priv->paned),
-				eel_gconf_get_integer (CONF_STATE_PANED_POSITION));
-	
-	if (eel_gconf_get_boolean (CONF_STATE_SHOW_BROWSER))
-		gtk_widget_show (source->priv->browser);
-	else
-		gtk_widget_hide (source->priv->browser);
+	key = rb_library_source_get_paned_key (source);
+	if (key)
+		gtk_paned_set_position (GTK_PANED (source->priv->paned),
+					eel_gconf_get_integer (key));
+
+	key = rb_source_get_browser_key (RB_SOURCE (source));
+	if (key && eel_gconf_get_boolean (key))
+			gtk_widget_show (source->priv->browser);
+		else
+			gtk_widget_hide (source->priv->browser);
 }
 
 static void
@@ -1336,6 +1340,17 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
+const char *
+rb_library_source_get_paned_key (RBLibrarySource *source)
+{
+	RBLibrarySourceClass *klass = RB_LIBRARY_SOURCE_GET_CLASS (source);
+
+	if (klass->impl_get_paned_key)
+		return klass->impl_get_paned_key (source);
+	else
+		return NULL;
+}
+
 static void
 push_multi_equals_query (RhythmDB *db, GPtrArray *query, guint propid, GList *items)
 {
@@ -1542,10 +1557,6 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 					    source->priv->cached_artists_model);
 		rb_property_view_set_model (source->priv->albums,
 					    source->priv->cached_albums_model);
-		g_object_ref (G_OBJECT (source->priv->cached_all_query));
-		g_object_ref (G_OBJECT (source->priv->cached_genres_model));
-		g_object_ref (G_OBJECT (source->priv->cached_artists_model));
-		g_object_ref (G_OBJECT (source->priv->cached_albums_model));
 	} else {
 		rb_debug ("query is not special");
 		source->priv->active_query = source->priv->model = 
