@@ -2,7 +2,7 @@
  *  arch-tag: Header for playlist source object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *  Copyright (C) 2003 Colin Walters <walters@debian.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,13 +23,13 @@
 #ifndef __RB_PLAYLIST_SOURCE_H
 #define __RB_PLAYLIST_SOURCE_H
 
-#include <bonobo/bonobo-ui-container.h>
 #include <libxml/tree.h>
 
-#include "rb-source.h"
-#include "rhythmdb.h"
 #include "rb-library-source.h"
 #include "rb-query-creator.h"
+#include "rb-shell.h"
+#include "rb-source.h"
+#include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 
 G_BEGIN_DECLS
@@ -57,22 +57,28 @@ typedef struct
 
 GType		rb_playlist_source_get_type	(void);
 
-RBSource *	rb_playlist_source_new		(RhythmDB *db, gboolean smart);
+RBSource *	rb_playlist_source_new		(RBShell *shell,
+						 gboolean smart, 
+						 gboolean local);
 
-RBSource *	rb_playlist_source_new_from_xml	(RhythmDB *db,
+RBSource *	rb_playlist_source_new_from_xml	(RBShell *shell,
 						 xmlNodePtr node);
 
-RhythmDBQueryModel *rb_playlist_source_get_model(RBPlaylistSource *source);
-
 void		rb_playlist_source_set_query	(RBPlaylistSource *source,
 						 GPtrArray *query,
 						 guint limit_count,
-						 guint limit_mb);
+						 guint limit_mb,
+						 guint limit_time,
+						 const char *sort_key,
+						 gint sort_order);
 
 void		rb_playlist_source_get_query	(RBPlaylistSource *source,
 						 GPtrArray **query,
 						 guint *limit_count,
-						 guint *limit_mb);
+						 guint *limit_mb,
+						 guint *limit_time,
+						 const char **sort_key,
+						 gint *sort_order);
 
 void		rb_playlist_source_save_playlist(RBPlaylistSource *source,
 						 const char *uri);
@@ -80,6 +86,8 @@ void		rb_playlist_source_save_playlist(RBPlaylistSource *source,
 void		rb_playlist_source_save_to_xml	(RBPlaylistSource *source,
 						 xmlNodePtr node);
 
+void		rb_playlist_source_burn_playlist(RBPlaylistSource *source);
+
 void		rb_playlist_source_add_entry	(RBPlaylistSource *source,
 						 RhythmDBEntry *entry);
 
@@ -89,6 +97,9 @@ void		rb_playlist_source_remove_entry	(RBPlaylistSource *source,
 void		rb_playlist_source_add_location	(RBPlaylistSource *source,
 						 const char *location);
 
+void            rb_playlist_source_add_locations (RBPlaylistSource *source,
+						  GList *locations);
+
 void		rb_playlist_source_remove_location	(RBPlaylistSource *source,
 						 	 const char *location);
 
