@@ -20,14 +20,7 @@
  *
  */
 
-#include <gtk/gtktreeview.h>
-
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktooltips.h>
-#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 #include <config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -43,7 +36,6 @@
 #include "rhythmdb-query-model.h"
 #include "rb-cell-renderer-pixbuf.h"
 #include "rb-cell-renderer-rating.h"
-#include "rb-string-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
@@ -71,6 +63,7 @@ static void rb_entry_view_get_property (GObject *object,
 				       GParamSpec *pspec);
 static void rb_entry_view_selection_changed_cb (GtkTreeSelection *selection,
 				               RBEntryView *view);
+static void rb_entry_view_grab_focus (GtkWidget *widget);
 static void rb_entry_view_row_activated_cb (GtkTreeView *treeview,
 			                   GtkTreePath *path,
 			                   GtkTreeViewColumn *column,
@@ -256,6 +249,7 @@ static void
 rb_entry_view_class_init (RBEntryViewClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -265,6 +259,8 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 	object_class->set_property = rb_entry_view_set_property;
 	object_class->get_property = rb_entry_view_get_property;
 
+	widget_class->grab_focus = rb_entry_view_grab_focus;
+
 	g_object_class_install_property (object_class,
 					 PROP_DB,
 					 g_param_spec_object ("db",
@@ -407,25 +403,27 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 static void
 rb_entry_view_init (RBEntryView *view)
 {
-	GtkWidget *dummy;
+	GtkIconTheme* icon_theme;
 	
 	view->priv = g_new0 (RBEntryViewPrivate, 1);
 
-	dummy = gtk_tree_view_new ();
-	view->priv->playing_pixbuf = gtk_widget_render_icon (dummy,
-							     RB_STOCK_PLAYING,
-							     GTK_ICON_SIZE_MENU,
-							     NULL);
+	icon_theme = gtk_icon_theme_get_default ();
 
-	view->priv->paused_pixbuf = gtk_widget_render_icon (dummy,
-							    RB_STOCK_PAUSED,
-							    GTK_ICON_SIZE_MENU,
-							    NULL);
-	view->priv->error_pixbuf = gtk_widget_render_icon (dummy,
-							   RB_STOCK_PLAYBACK_ERROR,
-							   GTK_ICON_SIZE_MENU,
-							   NULL);
-	gtk_widget_destroy (dummy);
+	view->priv->playing_pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                   			       "stock_volume-max", 
+                                   			       16,
+                                   			       0, 
+                                   			       NULL);
+	view->priv->paused_pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                   			      "stock_volume-0", 
+                                   			      16,
+                                   			      0, 
+                                   			      NULL);
+	view->priv->error_pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                   			     "stock_dialog-error", 
+                                   			     16,
+                                   			     0, 
+                                   			     NULL);
 
 	view->priv->propid_column_map = g_hash_table_new (NULL, NULL);
 	view->priv->column_sort_data_map = g_hash_table_new_full (NULL, NULL, NULL, g_free);
@@ -745,11 +743,24 @@ static gint
 rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
 {
+	const char *a_val;
+	const char *b_val;
 	gint ret;
 
 	/* Sort by album name */
-	ret = strcmp (rb_refstring_get_sort_key (a->album),
-		      rb_refstring_get_sort_key (b->album));
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
+
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			ret = 0;
+		else
+			ret = -1;
+	} else if (b_val == NULL)
+		ret = 1;
+	else
+		ret = strcmp (a_val, b_val);
+
 	if (ret != 0)
 		return ret;
 
@@ -762,36 +773,72 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 		return (a->tracknum < b->tracknum ? -1 : 1);
 
 	/* And finally by title */
-	return strcmp (rb_refstring_get_sort_key (a->title),
-		       rb_refstring_get_sort_key (b->title));
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_TITLE_SORT_KEY);
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_TITLE_SORT_KEY);
+
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			return 0;
+		else
+			return -1;
+	} else if (b_val == NULL)
+		return 1;
+	else
+		return strcmp (a_val, b_val);
 }
 
 static gint
 rb_entry_view_artist_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 				RBEntryView *view)
 {
+	const char *a_val;
+	const char *b_val;
 	gint ret;
 
-	ret = strcmp (rb_refstring_get_sort_key (a->artist),
-		      rb_refstring_get_sort_key (b->artist));
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			ret = 0;
+		else
+			ret = -1;
+	} else if (b_val == NULL)
+		ret = 1;
+	else
+		ret = strcmp (a_val, b_val);
+
 	if (ret != 0)
 		return ret;
-
-	return rb_entry_view_album_sort_func (a, b, view);
+	else
+		return rb_entry_view_album_sort_func (a, b, view);
 }
 
 static gint
 rb_entry_view_genre_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
 {
+	const char *a_val;
+	const char *b_val;
 	gint ret;
 
-	ret = strcmp (rb_refstring_get_sort_key (a->genre),
-		      rb_refstring_get_sort_key (b->genre));
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_GENRE_SORT_KEY);
+	b_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_GENRE_SORT_KEY);
+
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			ret = 0;
+		else
+			ret = -1;
+	} else if (b_val == NULL)
+		ret = 1;
+	else
+		ret = strcmp (a_val, b_val);
+
 	if (ret != 0)
 		return ret;
-
-	return rb_entry_view_artist_sort_func (a, b, view);
+	else
+		return rb_entry_view_artist_sort_func (a, b, view);
 }
 
 static gint
@@ -836,15 +883,20 @@ rb_entry_view_string_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 				struct RBEntryViewCellDataFuncData *data)
 {
 	const char *a_val;
-	const char *b_val;	
-	gint ret;
+	const char *b_val;
 
 	a_val = rhythmdb_entry_get_string (a, data->propid);
 	b_val = rhythmdb_entry_get_string (b, data->propid);
 
-	ret = strcmp (a_val, b_val);
-
-	return ret;
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			return 0;
+		else
+			return -1;
+	} else if (b_val == NULL)
+		return 1;
+	else
+		return strcmp (a_val, b_val);
 }
 
 static void
@@ -933,17 +985,20 @@ rb_entry_view_duration_cell_data_func (GtkTreeViewColumn *column, GtkCellRendere
 {
 	RhythmDBEntry *entry;
 	char *str;
-	int minutes, seconds;
+	int hours, minutes, seconds;
 
 	entry = entry_from_tree_iter (data->view, iter);
 
-	minutes = entry->duration / 60;
+	hours = entry->duration / (60 * 60);
+	minutes = (entry->duration - (hours * 60 * 60)) / 60;
 	seconds = entry->duration % 60;
 
-	if (minutes == 0 && seconds == 0)
+	if (hours == 0 && minutes == 0 && seconds == 0)
 		str = g_strdup (_("Unknown"));
-	else
+	else if (hours == 0)
 		str = g_strdup_printf (_("%d:%02d"), minutes, seconds);
+	else
+		str = g_strdup_printf (_("%d:%02d:%02d"), hours, minutes, seconds);
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
 
@@ -1087,15 +1142,18 @@ rb_entry_view_set_sorting_type (RBEntryView *view, const char *sorttype)
 void 
 rb_entry_view_get_sorting_order (RBEntryView *view, const char **column_name, gint *sort_order)
 {
-	*column_name = g_strdup(view->priv->sorting_column_name);
+	*column_name = g_strdup (view->priv->sorting_column_name);
 	*sort_order = view->priv->sorting_order;
 }
 
 void
 rb_entry_view_set_sorting_order (RBEntryView *view, const char *column_name, gint sort_order)
 {
-	g_free(view->priv->sorting_column_name);
-	view->priv->sorting_column_name = g_strdup(column_name);
+	if (column_name == NULL)
+		return;
+
+	g_free (view->priv->sorting_column_name);
+	view->priv->sorting_column_name = g_strdup (column_name);
 	view->priv->sorting_order = sort_order;
 	
 	rb_entry_view_sync_sorting (view);
@@ -1197,7 +1255,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_long_cell_data_func;
 		sort_func = (GCompareDataFunc) rb_entry_view_track_sort_func;
 		real_sort_data = view;
-		title = _("Tra_ck");
+		title = _("Trac_k");
 		key = "Track";
 		break;
 	case RB_ENTRY_VIEW_COL_TITLE:
@@ -1228,7 +1286,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		sort_data->propid = RHYTHMDB_PROP_ALBUM_SORT_KEY;
 		sort_func = (GCompareDataFunc) rb_entry_view_album_sort_func;
 		real_sort_data = view;
-		title = _("A_lbum");
+		title = _("_Album");
 		key = "Album";
 		ellipsize = TRUE;
 		break;
@@ -1239,7 +1297,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		sort_data->propid = RHYTHMDB_PROP_GENRE_SORT_KEY;
 		sort_func = (GCompareDataFunc) rb_entry_view_genre_sort_func;
 		real_sort_data = view;
-		title = _("Ge_nre");
+		title = _("_Genre");
 		key = "Genre";
 		ellipsize = TRUE;
 		break;
@@ -1249,7 +1307,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_duration_cell_data_func;
 		sort_data->propid = cell_data->propid;
 		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
-		title = _("Ti_me");
+		title = _("Tim_e");
 		key = "Time";
 		break;
 #if 0
@@ -1279,9 +1337,18 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;				
 		sort_data->propid = RHYTHMDB_PROP_LAST_PLAYED;
 		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
-		title = _("L_ast Played");
+		title = _("_Last Played");
 		key = "LastPlayed";
 		break;
+	case RB_ENTRY_VIEW_COL_FIRST_SEEN:
+		propid = RHYTHMDB_PROP_FIRST_SEEN;
+		cell_data->propid = RHYTHMDB_PROP_FIRST_SEEN_STR;
+		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;				
+		sort_data->propid = RHYTHMDB_PROP_FIRST_SEEN;
+		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
+		title = _("_Date Added");
+		key = "FirstSeen";
+		break;
 	case RB_ENTRY_VIEW_COL_RATING:
 	default:
 		g_assert_not_reached ();
@@ -1418,6 +1485,7 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 		GtkTreeViewColumn *column;
 		GtkTooltips *tooltip;
 		GtkCellRenderer *renderer;
+		GtkWidget *image_widget;
 		gint width;
 
 		tooltip = gtk_tooltips_new ();
@@ -1431,7 +1499,16 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 							 rb_entry_view_playing_cell_data_func,
 							 view,
 							 NULL);
+
+		image_widget = gtk_image_new_from_pixbuf (view->priv->playing_pixbuf);
+		g_object_ref (G_OBJECT (image_widget));
+		gtk_object_sink (GTK_OBJECT (image_widget));
+		gtk_tree_view_column_set_widget (column, image_widget);
+		gtk_widget_show (image_widget);
+
+
 		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
+		gtk_tree_view_column_set_clickable (column, FALSE);
 		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, NULL);
 		gtk_tree_view_column_set_fixed_width (column, width + 5);
 		gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
@@ -1439,6 +1516,9 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 					  "pixbuf-clicked",
 					  G_CALLBACK (rb_entry_view_pixbuf_clicked_cb),
 					  G_OBJECT (view));
+		g_signal_connect_object (G_OBJECT (column), "clicked",
+				 G_CALLBACK (rb_entry_view_column_clicked_cb),
+				 view, 0);
 
 		gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltip), GTK_WIDGET (column->button),
 				       _("Now Playing"), NULL);
@@ -1499,15 +1579,7 @@ rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 
 	g_value_init (&value, G_TYPE_DOUBLE);
 	g_value_set_double (&value, rating);
-	rhythmdb_entry_sync (view->priv->db, entry, RHYTHMDB_PROP_RATING,
-			    &value);
-	g_value_unset (&value);
-
-	/* since the user changed the rating, stop auto-rating */
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, FALSE);
-	rhythmdb_entry_sync (view->priv->db, entry, RHYTHMDB_PROP_AUTO_RATE,
-			    &value);
+	rhythmdb_entry_set (view->priv->db, entry, RHYTHMDB_PROP_RATING, &value);
 	g_value_unset (&value);
 
 	rhythmdb_commit (view->priv->db);
@@ -2091,9 +2163,19 @@ void
 rb_entry_view_set_playing (RBEntryView *view,
 			   gboolean playing)
 {
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
 	g_return_if_fail (RB_IS_ENTRY_VIEW (view));
 
-	view->priv->playing = TRUE;
+	view->priv->playing = playing;
+	/* Redraw the treeview */
+	if (view->priv->playing_entry) {
+		rhythmdb_query_model_entry_to_iter (view->priv->playing_model, view->priv->playing_entry, &iter);
+		path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->playing_model), &iter);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (view->priv->playing_model), path, &iter);
+  		gtk_tree_path_free (path);
+	}
 }
 
 gboolean
@@ -2106,3 +2188,11 @@ rb_entry_view_poll_model (RBEntryView *view)
 
 	return rhythmdb_query_model_poll (view->priv->model, &timeout);
 }
+
+static void
+rb_entry_view_grab_focus (GtkWidget *widget)
+{
+	RBEntryView *view = RB_ENTRY_VIEW (widget);
+
+	gtk_widget_grab_focus (GTK_WIDGET (view->priv->treeview));
+}
