@@ -51,6 +51,7 @@ typedef enum {
 	RB_ENTRY_VIEW_COL_RATING,
 	RB_ENTRY_VIEW_COL_PLAY_COUNT,
 	RB_ENTRY_VIEW_COL_LAST_PLAYED,
+	RB_ENTRY_VIEW_COL_FIRST_SEEN
 } RBEntryViewColumn;
 
 typedef struct RBEntryViewPrivate RBEntryViewPrivate;
@@ -68,6 +69,7 @@ typedef struct
 
 	void (*entry_added)		(RBEntryView *view, RhythmDBEntry *entry);
 	void (*entry_deleted)		(RBEntryView *view, RhythmDBEntry *entry);
+	void (*entries_replaced)	(RBEntryView *view);
 
 	void (*entry_selected)          (RBEntryView *view, RhythmDBEntry *entry);
 	void (*entry_activated)         (RBEntryView *view, RhythmDBEntry *entry);
@@ -142,12 +144,17 @@ void		rb_entry_view_enable_drag_source	(RBEntryView *view,
 							 int n_targets);
 RhythmDBEntry *	rb_entry_view_get_random_entry		(RBEntryView *view);
 
+void		rb_entry_view_get_sorting_order		(RBEntryView *view, const char **column_name, gint *sort_order);
+void		rb_entry_view_set_sorting_order		(RBEntryView *view, const char *column_name, gint sort_order);
+/* deal with the sorting order as a composite string */
 const char *	rb_entry_view_get_sorting_type		(RBEntryView *view);
+ void		rb_entry_view_set_sorting_type		(RBEntryView *view, const char *sorttype);
+
+/** Should be called by sources before resorting the query model. */
+void		rb_entry_view_set_resorting		(RBEntryView *view);
 
 gboolean	rb_entry_view_poll_model		(RBEntryView *view);
 
-void		rb_entry_view_freeze			(RBEntryView *view);
-void		rb_entry_view_thaw			(RBEntryView *view);
 G_END_DECLS
 
 #endif /* __RB_ENTRY_VIEW_H */
