@@ -84,7 +84,6 @@ struct RBHeaderPrivate
 
 	GtkWidget *timeframe;
 	GtkWidget *timeline;
-	gboolean timeline_shown;
 	GtkWidget *scale;
 	GtkAdjustment *adjustment;
 	gboolean slider_dragging;
@@ -312,6 +311,8 @@ rb_header_init (RBHeader *player)
 	gtk_container_add (GTK_CONTAINER (align), player->priv->elapsed);
 	gtk_box_pack_start (GTK_BOX (scalebox), align, FALSE, TRUE, 0);
 	align = player->priv->timeframe = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
+	gtk_container_add (GTK_CONTAINER (player->priv->timeframe), player->priv->timeline);
+	gtk_widget_set_sensitive (player->priv->timeline, FALSE);
 
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
 
@@ -455,10 +456,10 @@ rb_header_get_duration (RBHeader *player)
 }
 
 static char *
-info_url (guint opt, const char *data)
+sanitize_string (const char *data)
 {
-	char *ret, *d;
-
+	char *ret;
+	
 	/*
 	 * netlabels often put URLs (or domain names) in the 'album' field
 	 * of their releases; since there are no artist names in AMG that
@@ -483,35 +484,38 @@ info_url (guint opt, const char *data)
 		else
 			ret = g_strdup_printf ("http://%s", data);
 	} else {
-		char *r;
-
-		d = g_strdup (data);
-		r = d;
-		while (*r != '\0') {
-			if (*r == ' ')
-				*r = '+';
-			r++;
-		}
-
-		ret = g_strdup_printf ("http://www.allmusic.com/cg/amg.dll?p=amg&opt1=%d&sql=%s", opt, d);
-		g_free (d);
+		ret = g_strdup (data);
 	}
+	return g_strstrip (ret);
+}
+
+static char *
+info_url (const char *artist, const char *album)
+{
+	char *escaped_artist;
+	char *sanitized_artist;
+	char *ret;
+
+	sanitized_artist = sanitize_string (artist);
+	escaped_artist = gnome_vfs_escape_string (sanitized_artist);
+	g_free (sanitized_artist);
+	if (album) {
+		char *sanitized_album;
+		char *escaped_album;
+		sanitized_album = sanitize_string (album);
+		escaped_album = gnome_vfs_escape_string (sanitized_album);
+		g_free (sanitized_album);
+		ret = g_strdup_printf ("http://www.last.fm/music/%s/%s",
+				       escaped_artist, escaped_album);
+		g_free (escaped_album);
+	} else {
+		ret = g_strdup_printf ("http://www.last.fm/music/%s", escaped_artist);
+	}
+	g_free (escaped_artist);
 
 	return ret;
 }
 
-static char *
-album_info_url (const char *album)
-{
-	return info_url (2, album);
-}
-
-static char *
-artist_info_url (const char *album)
-{
-	return info_url (1, album);
-}
-
 void
 rb_header_sync (RBHeader *player)
 {
@@ -546,7 +550,7 @@ rb_header_sync (RBHeader *player)
 
 			rb_debug ("artist: %s album: %s", artist, album);
 
-			tmp = album_info_url (album);
+			tmp = info_url (artist, album);
 			gnome_href_set_url (player->priv->displaybox->album, tmp);
 			escaped = g_markup_escape_text (album, -1);
 			gnome_href_set_text (player->priv->displaybox->album, escaped);
@@ -559,7 +563,7 @@ rb_header_sync (RBHeader *player)
 					      _("Get information on this album from the web"), 
 					      NULL);
 			
-			tmp = artist_info_url (artist);
+			tmp = info_url (artist, NULL);
 			gnome_href_set_url (player->priv->displaybox->artist, tmp);
 			escaped = g_markup_escape_text (artist, -1);
 			gnome_href_set_text (player->priv->displaybox->artist, escaped);
@@ -656,17 +660,7 @@ static void
 rb_header_set_show_timeline (RBHeader *player,
 			     gboolean show)
 {
-	if (player->priv->timeline_shown == show)
-		return;
-
-	player->priv->timeline_shown = show;
-
-	if (show == FALSE)
-		gtk_container_remove (GTK_CONTAINER (player->priv->timeframe), player->priv->timeline);
-	else {
-		gtk_container_add (GTK_CONTAINER (player->priv->timeframe), player->priv->timeline);
-		gtk_widget_show_all (player->priv->timeline);
-	}
+	gtk_widget_set_sensitive (player->priv->timeline, show);
 }
 
 gboolean
@@ -844,29 +838,49 @@ slider_changed_callback (GtkWidget *widget,
 char *
 rb_header_get_elapsed_string (RBHeader *player)
 {
-	int seconds = 0, minutes = 0, seconds2 = -1, minutes2 = -1;
+	int seconds = 0, minutes = 0, hours = 0, seconds2 = -1, minutes2 = -1, hours2 = -1;
 
 	if (player->priv->state->elapsed > 0) {
-		minutes = player->priv->state->elapsed / 60;
+		hours = player->priv->state->elapsed / (60 * 60);
+		minutes = (player->priv->state->elapsed - (hours * 60 * 60)) / 60;
 		seconds = player->priv->state->elapsed % 60;
 	}
 
 	if (player->priv->state->duration > 0) {
-		minutes2 = player->priv->state->duration / 60;
+		hours2 = player->priv->state->duration / (60 * 60);
+		minutes2 = (player->priv->state->duration - (hours2 * 60 * 60)) / 60;
 		seconds2 = player->priv->state->duration % 60;
 		if (eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY)) {
-			return g_strdup_printf (_("%d:%02d of %d:%02d"), minutes, seconds, minutes2, seconds2);
+			if (hours == 0 && hours2 == 0)
+				return g_strdup_printf (_("%d:%02d of %d:%02d"),
+							minutes, seconds,
+							minutes2, seconds2);
+			else
+				return g_strdup_printf (_("%d:%02d:%02d of %d:%02d:%02d"),
+							hours, minutes, seconds,
+							hours2, minutes2, seconds2);
 		} else {
 			int remaining = player->priv->state->duration - player->priv->state->elapsed;
-			int remaining_minutes = remaining / 60;
+			int remaining_hours = remaining / (60 * 60);
+			int remaining_minutes = (remaining - (remaining_hours * 60 * 60)) / 60;
 			/* remaining could conceivably be negative. This would
 			 * be a bug, but the elapsed time will display right
 			 * with the abs(). */
 			int remaining_seconds = abs (remaining % 60);
-			return g_strdup_printf (_("%d:%02d of %d:%02d remaining"), remaining_minutes, remaining_seconds, minutes2, seconds2);
+			if (hours2 == 0)
+				return g_strdup_printf (_("%d:%02d of %d:%02d remaining"),
+							remaining_minutes, remaining_seconds,
+							minutes2, seconds2);
+			else
+				return g_strdup_printf (_("%d:%02d:%02d of %d:%02d:%02d remaining"),
+							remaining_hours, remaining_minutes, remaining_seconds,
+							hours2, minutes2, seconds2);
 		}
 	} else {
-		return g_strdup_printf (_("%d:%02d"), minutes, seconds);
+		if (hours == 0)
+			return g_strdup_printf (_("%d:%02d"), minutes, seconds);
+		else
+			return g_strdup_printf (_("%d:%02d:%02d"), hours, minutes, seconds);
 	}
 }
 
